@@ -46,15 +46,15 @@
 #define ReAlloc(P, SIZ)		erts_realloc(ERTS_ALC_T_BPD, (P), (SZ))
 #define Free(P)			erts_free(ERTS_ALC_T_BPD, (P))
 
-#if defined(ERTS_ENABLE_LOCK_CHECK) && defined(ERTS_SMP)
-#  define ERTS_SMP_REQ_PROC_MAIN_LOCK(P) \
+#if defined(ERTS_ENABLE_LOCK_CHECK)
+#  define ERTS_REQ_PROC_MAIN_LOCK(P) \
       if ((P)) erts_proc_lc_require_lock((P), ERTS_PROC_LOCK_MAIN,\
 					 __FILE__, __LINE__)
-#  define ERTS_SMP_UNREQ_PROC_MAIN_LOCK(P) \
+#  define ERTS_UNREQ_PROC_MAIN_LOCK(P) \
       if ((P)) erts_proc_lc_unrequire_lock((P), ERTS_PROC_LOCK_MAIN)
 #else
-#  define ERTS_SMP_REQ_PROC_MAIN_LOCK(P)
-#  define ERTS_SMP_UNREQ_PROC_MAIN_LOCK(P)
+#  define ERTS_REQ_PROC_MAIN_LOCK(P)
+#  define ERTS_UNREQ_PROC_MAIN_LOCK(P)
 #endif
 
 #define ERTS_BPF_LOCAL_TRACE       0x01
@@ -73,11 +73,9 @@ extern BeamInstr beam_return_trace[1];      /* OpCode(i_return_trace) */
 extern BeamInstr beam_exception_trace[1];   /* OpCode(i_exception_trace) */
 extern BeamInstr beam_return_time_trace[1]; /* OpCode(i_return_time_trace) */
 
-erts_smp_atomic32_t erts_active_bp_index;
-erts_smp_atomic32_t erts_staging_bp_index;
-#ifdef ERTS_DIRTY_SCHEDULERS
-erts_smp_mtx_t erts_dirty_bp_ix_mtx;
-#endif
+erts_atomic32_t erts_active_bp_index;
+erts_atomic32_t erts_staging_bp_index;
+erts_mtx_t erts_dirty_bp_ix_mtx;
 
 /*
  * Inlined helpers
@@ -94,22 +92,18 @@ acquire_bp_sched_ix(Process *c_p)
 {
     ErtsSchedulerData *esdp = erts_proc_sched_data(c_p);
     ASSERT(esdp);
-#ifdef ERTS_DIRTY_SCHEDULERS
     if (ERTS_SCHEDULER_IS_DIRTY(esdp)) {
-	erts_smp_mtx_lock(&erts_dirty_bp_ix_mtx);
+	erts_mtx_lock(&erts_dirty_bp_ix_mtx);
         return (Uint32) erts_no_schedulers;
     }
-#endif
     return (Uint32) esdp->no - 1;
 }
 
 static ERTS_INLINE void
 release_bp_sched_ix(Uint32 ix)
 {
-#ifdef ERTS_DIRTY_SCHEDULERS
     if (ix == (Uint32) erts_no_schedulers)
-        erts_smp_mtx_unlock(&erts_dirty_bp_ix_mtx);
-#endif
+        erts_mtx_unlock(&erts_dirty_bp_ix_mtx);
 }
 
 
@@ -162,12 +156,10 @@ static void bp_hash_delete(bp_time_hash_t *hash);
 
 void 
 erts_bp_init(void) {
-    erts_smp_atomic32_init_nob(&erts_active_bp_index, 0);
-    erts_smp_atomic32_init_nob(&erts_staging_bp_index, 1);
-#ifdef ERTS_DIRTY_SCHEDULERS
-    erts_smp_mtx_init(&erts_dirty_bp_ix_mtx, "dirty_break_point_index", NIL,
+    erts_atomic32_init_nob(&erts_active_bp_index, 0);
+    erts_atomic32_init_nob(&erts_staging_bp_index, 1);
+    erts_mtx_init(&erts_dirty_bp_ix_mtx, "dirty_break_point_index", NIL,
         ERTS_LOCK_FLAGS_PROPERTY_STATIC | ERTS_LOCK_FLAGS_CATEGORY_DEBUG);
-#endif
 }
 
 
@@ -211,7 +203,7 @@ erts_bp_match_functions(BpFunctions* f, ErtsCodeMFA *mfa, int specified)
 	for (fi = 0; fi < num_functions; fi++) {
 
 	    ci = code_hdr->functions[fi];
-	    ASSERT(ci->op == (BeamInstr) BeamOp(op_i_func_info_IaaI));
+	    ASSERT(BeamIsOpCode(ci->op, op_i_func_info_IaaI));
 	    if (erts_is_function_native(ci)) {
 		continue;
 	    }
@@ -273,11 +265,11 @@ erts_bp_match_export(BpFunctions* f, ErtsCodeMFA *mfa, int specified)
 
 	pc = ep->beam;
 	if (ep->addressv[code_ix] == pc) {
-	    if ((*pc == (BeamInstr) em_apply_bif ||
-		 *pc == (BeamInstr) em_call_error_handler)) {
-		continue;
+	    if (BeamIsOpCode(*pc, op_apply_bif) ||
+                BeamIsOpCode(*pc, op_call_error_handler)) {
+                continue;
 	    }
-	    ASSERT(*pc == (BeamInstr) BeamOp(op_i_generic_breakpoint));
+	    ASSERT(BeamIsOpCode(*pc, op_i_generic_breakpoint));
 	} else if (erts_is_function_native(erts_code_to_codeinfo(ep->addressv[code_ix]))) {
 	    continue;
 	}
@@ -306,7 +298,7 @@ erts_consolidate_bp_data(BpFunctions* f, int local)
     Uint i;
     Uint n = f->matched;
 
-    ERTS_SMP_LC_ASSERT(erts_has_code_write_permission());
+    ERTS_LC_ASSERT(erts_has_code_write_permission());
 
     for (i = 0; i < n; i++) {
 	consolidate_bp_data(fs[i].mod, fs[i].ci, local);
@@ -318,7 +310,7 @@ erts_consolidate_bif_bp_data(void)
 {
     int i;
 
-    ERTS_SMP_LC_ASSERT(erts_has_code_write_permission());
+    ERTS_LC_ASSERT(erts_has_code_write_permission());
     for (i = 0; i < BIF_SIZE; i++) {
 	Export *ep = bif_export[i];
 	consolidate_bp_data(0, &ep->info, 0);
@@ -374,8 +366,8 @@ consolidate_bp_data(Module* modp, ErtsCodeInfo *ci, int local)
 	    }
 	    ASSERT(modp->curr.num_breakpoints >= 0);
 	    ASSERT(modp->curr.num_traced_exports >= 0);
-	    ASSERT(*erts_codeinfo_to_code(ci) !=
-                   (BeamInstr) BeamOp(op_i_generic_breakpoint));
+	    ASSERT(! BeamIsOpCode(*erts_codeinfo_to_code(ci),
+                                  op_i_generic_breakpoint));
 	}
 	ci->u.gen_bp = NULL;
 	Free(g);
@@ -393,17 +385,17 @@ consolidate_bp_data(Module* modp, ErtsCodeInfo *ci, int local)
     }
     if (flags & ERTS_BPF_META_TRACE) {
 	dst->meta_tracer = src->meta_tracer;
-	erts_smp_refc_inc(&dst->meta_tracer->refc, 1);
+	erts_refc_inc(&dst->meta_tracer->refc, 1);
 	dst->meta_ms = src->meta_ms;
 	MatchSetRef(dst->meta_ms);
     }
     if (flags & ERTS_BPF_COUNT) {
 	dst->count = src->count;
-	erts_smp_refc_inc(&dst->count->refc, 1);
+	erts_refc_inc(&dst->count->refc, 1);
     }
     if (flags & ERTS_BPF_TIME_TRACE) {
 	dst->time = src->time;
-	erts_smp_refc_inc(&dst->time->refc, 1);
+	erts_refc_inc(&dst->time->refc, 1);
 	ASSERT(dst->time->hash);
     }
 }
@@ -414,8 +406,8 @@ erts_commit_staged_bp(void)
     ErtsBpIndex staging = erts_staging_bp_ix();
     ErtsBpIndex active = erts_active_bp_ix();
 
-    erts_smp_atomic32_set_nob(&erts_active_bp_index, staging);
-    erts_smp_atomic32_set_nob(&erts_staging_bp_index, active);
+    erts_atomic32_set_nob(&erts_active_bp_index, staging);
+    erts_atomic32_set_nob(&erts_staging_bp_index, active);
 }
 
 void
@@ -423,13 +415,15 @@ erts_install_breakpoints(BpFunctions* f)
 {
     Uint i;
     Uint n = f->matched;
-    BeamInstr br = (BeamInstr) BeamOp(op_i_generic_breakpoint);
+    BeamInstr br = BeamOpCodeAddr(op_i_generic_breakpoint);
 
     for (i = 0; i < n; i++) {
 	ErtsCodeInfo* ci = f->matching[i].ci;
-        BeamInstr *pc = erts_codeinfo_to_code(ci);
 	GenericBp* g = ci->u.gen_bp;
-	if (*pc != br && g) {
+        BeamInstr volatile *pc = erts_codeinfo_to_code(ci);
+        BeamInstr instr = *pc;
+
+	if (!BeamIsOpCode(instr, op_i_generic_breakpoint) && g) {
 	    Module* modp = f->matching[i].mod;
 
 	    /*
@@ -443,11 +437,16 @@ erts_install_breakpoints(BpFunctions* f)
 	    /*
 	     * The following write is not protected by any lock. We
 	     * assume that the hardware guarantees that a write of an
-	     * aligned word-size (or half-word) writes is atomic
-	     * (i.e. that other processes executing this code will not
-	     * see a half pointer).
+	     * aligned word-size writes is atomic (i.e. that other
+	     * processes executing this code will not see a half
+	     * pointer).
+             *
+             * The contents of *pc is marked 'volatile' to ensure that
+             * the compiler will do a single full-word write, and not
+             * try any fancy optimizations to write a half word.
 	     */
-	    *pc = br;
+            instr = BeamSetCodeAddr(instr, br);
+            *pc = instr;
 	    modp->curr.num_breakpoints++;
 	}
     }
@@ -468,7 +467,7 @@ static void
 uninstall_breakpoint(ErtsCodeInfo *ci)
 {
     BeamInstr *pc = erts_codeinfo_to_code(ci);
-    if (*pc == (BeamInstr) BeamOp(op_i_generic_breakpoint)) {
+    if (BeamIsOpCode(*pc, op_i_generic_breakpoint)) {
 	GenericBp* g = ci->u.gen_bp;
 	if (g->data[erts_active_bp_ix()].flags == 0) {
 	    /*
@@ -575,7 +574,7 @@ erts_clear_mtrace_bif(ErtsCodeInfo *ci)
 void
 erts_clear_debug_break(BpFunctions* f)
 {
-    ERTS_SMP_LC_ASSERT(erts_smp_thr_progress_is_blocking());
+    ERTS_LC_ASSERT(erts_thr_progress_is_blocking());
     clear_break(f, ERTS_BPF_DEBUG);
 }
 
@@ -603,7 +602,7 @@ erts_clear_module_break(Module *modp) {
     Uint n;
     Uint i;
 
-    ERTS_SMP_LC_ASSERT(erts_smp_thr_progress_is_blocking());
+    ERTS_LC_ASSERT(erts_thr_progress_is_blocking());
     ASSERT(modp);
     code_hdr = modp->curr.code_hdr;
     if (!code_hdr) {
@@ -633,7 +632,7 @@ erts_clear_module_break(Module *modp) {
 void
 erts_clear_export_break(Module* modp, ErtsCodeInfo *ci)
 {
-    ERTS_SMP_LC_ASSERT(erts_smp_thr_progress_is_blocking());
+    ERTS_LC_ASSERT(erts_thr_progress_is_blocking());
 
     clear_function_break(ci, ERTS_BPF_ALL);
     erts_commit_staged_bp();
@@ -656,12 +655,12 @@ static void fixup_cp_before_trace(Process *c_p, int *return_to_trace)
 {
     Eterm *cpp, *E = c_p->stop;
     BeamInstr w = *c_p->cp;
-    if (w == (BeamInstr) BeamOp(op_return_trace)) {
+    if (BeamIsOpCode(w, op_return_trace)) {
         cpp = &E[2];
-    } else if (w == (BeamInstr) BeamOp(op_i_return_to_trace)) {
+    } else if (BeamIsOpCode(w, op_i_return_to_trace)) {
         *return_to_trace = 1;
         cpp = &E[0];
-    } else if (w == (BeamInstr) BeamOp(op_i_return_time_trace)) {
+    } else if (BeamIsOpCode(w, op_i_return_time_trace)) {
         cpp = &E[0];
     } else {
         cpp = NULL;
@@ -669,12 +668,12 @@ static void fixup_cp_before_trace(Process *c_p, int *return_to_trace)
     if (cpp) {
         for (;;) {
             BeamInstr w = *cp_val(*cpp);
-            if (w == (BeamInstr) BeamOp(op_return_trace)) {
+            if (BeamIsOpCode(w, op_return_trace)) {
                 cpp += 3;
-            } else if (w == (BeamInstr) BeamOp(op_i_return_to_trace)) {
+            } else if (BeamIsOpCode(w, op_i_return_to_trace)) {
                 *return_to_trace = 1;
                 cpp += 1;
-            } else if (w == (BeamInstr) BeamOp(op_i_return_time_trace)) {
+            } else if (BeamIsOpCode(w, op_i_return_time_trace)) {
                 cpp += 2;
             } else {
                 break;
@@ -693,7 +692,7 @@ erts_generic_breakpoint(Process* c_p, ErtsCodeInfo *info, Eterm* reg)
     Uint bp_flags;
     ErtsBpIndex ix = erts_active_bp_ix();
 
-    ASSERT(info->op == (BeamInstr) BeamOp(op_i_func_info_IaaI));
+    ASSERT(BeamIsOpCode(info->op, op_i_func_info_IaaI));
 
     g = info->u.gen_bp;
     bp = &g->data[ix];
@@ -722,12 +721,12 @@ erts_generic_breakpoint(Process* c_p, ErtsCodeInfo *info, Eterm* reg)
     if (bp_flags & ERTS_BPF_META_TRACE) {
 	ErtsTracer old_tracer, new_tracer;
 
-	old_tracer = erts_smp_atomic_read_nob(&bp->meta_tracer->tracer);
+	old_tracer = erts_atomic_read_nob(&bp->meta_tracer->tracer);
 
 	new_tracer = do_call_trace(c_p, info, reg, 1, bp->meta_ms, old_tracer);
 
 	if (!ERTS_TRACER_COMPARE(new_tracer, old_tracer)) {
-            if (old_tracer == erts_smp_atomic_cmpxchg_acqb(
+            if (old_tracer == erts_atomic_cmpxchg_acqb(
                     &bp->meta_tracer->tracer,
                     (erts_aint_t)new_tracer,
                     (erts_aint_t)old_tracer)) {
@@ -739,16 +738,16 @@ erts_generic_breakpoint(Process* c_p, ErtsCodeInfo *info, Eterm* reg)
     }
 
     if (bp_flags & ERTS_BPF_COUNT_ACTIVE) {
-	erts_smp_atomic_inc_nob(&bp->count->acount);
+	erts_atomic_inc_nob(&bp->count->acount);
     }
 
     if (bp_flags & ERTS_BPF_TIME_TRACE_ACTIVE) {
 	Eterm w;
 	erts_trace_time_call(c_p, info, bp->time);
 	w = (BeamInstr) *c_p->cp;
-	if (! (w == (BeamInstr) BeamOp(op_i_return_time_trace) ||
-	       w == (BeamInstr) BeamOp(op_return_trace) ||
-	       w == (BeamInstr) BeamOp(op_i_return_to_trace)) ) {
+	if (! (BeamIsOpCode(w, op_i_return_time_trace) ||
+	       BeamIsOpCode(w, op_return_trace) ||
+               BeamIsOpCode(w, op_i_return_to_trace)) ) {
 	    Eterm* E = c_p->stop;
 	    ASSERT(c_p->htop <= E && E <= c_p->hend);
 	    if (E - 2 < c_p->htop) {
@@ -768,7 +767,7 @@ erts_generic_breakpoint(Process* c_p, ErtsCodeInfo *info, Eterm* reg)
     }
 
     if (bp_flags & ERTS_BPF_DEBUG) {
-	return (BeamInstr) BeamOp(op_i_debug_breakpoint);
+	return BeamOpCodeAddr(op_i_debug_breakpoint);
     } else {
 	return g->orig_instr;
     }
@@ -797,7 +796,7 @@ erts_bif_trace(int bif_index, Process* p, Eterm* args, BeamInstr* I)
     Uint bp_flags = 0;
     int return_to_trace = 0;
 
-    ERTS_SMP_CHK_HAVE_ONLY_MAIN_PROC_LOCK(p);
+    ERTS_CHK_HAVE_ONLY_MAIN_PROC_LOCK(p);
 
     g = ep->info.u.gen_bp;
     if (g) {
@@ -823,7 +822,7 @@ erts_bif_trace(int bif_index, Process* p, Eterm* args, BeamInstr* I)
     if (bp_flags & ERTS_BPF_META_TRACE) {
 	ErtsTracer old_tracer;
 
-        meta_tracer = erts_smp_atomic_read_nob(&bp->meta_tracer->tracer);
+        meta_tracer = erts_atomic_read_nob(&bp->meta_tracer->tracer);
         old_tracer = meta_tracer;
 	flags_meta = erts_call_trace(p, &ep->info, bp->meta_ms, args,
 				     0, &meta_tracer);
@@ -831,7 +830,7 @@ erts_bif_trace(int bif_index, Process* p, Eterm* args, BeamInstr* I)
 	if (!ERTS_TRACER_COMPARE(old_tracer, meta_tracer)) {
             ErtsTracer new_tracer = erts_tracer_nil;
             erts_tracer_update(&new_tracer, meta_tracer);
-	    if (old_tracer == erts_smp_atomic_cmpxchg_acqb(
+	    if (old_tracer == erts_atomic_cmpxchg_acqb(
                     &bp->meta_tracer->tracer,
                     (erts_aint_t)new_tracer,
                     (erts_aint_t)old_tracer)) {
@@ -958,9 +957,9 @@ erts_bif_trace_epilogue(Process *p, Eterm result, int applying,
 		}
 	    }
 	    if ((flags_meta|flags) & MATCH_SET_EXCEPTION_TRACE) {
-		erts_smp_proc_lock(p, ERTS_PROC_LOCKS_ALL_MINOR);
+		erts_proc_lock(p, ERTS_PROC_LOCKS_ALL_MINOR);
 		ERTS_TRACE_FLAGS(p) |= F_EXCEPTION_TRACE;
-		erts_smp_proc_unlock(p, ERTS_PROC_LOCKS_ALL_MINOR);
+		erts_proc_unlock(p, ERTS_PROC_LOCKS_ALL_MINOR);
 	    }
 	}
     } else {
@@ -983,7 +982,7 @@ erts_bif_trace_epilogue(Process *p, Eterm result, int applying,
 	    }
 	}
     }
-    ERTS_SMP_CHK_HAVE_ONLY_MAIN_PROC_LOCK(p);
+    ERTS_CHK_HAVE_ONLY_MAIN_PROC_LOCK(p);
     return result;
 }
 
@@ -999,9 +998,9 @@ do_call_trace(Process* c_p, ErtsCodeInfo* info, Eterm* reg,
 
     fixup_cp_before_trace(c_p, &return_to_trace);
 
-    ERTS_SMP_UNREQ_PROC_MAIN_LOCK(c_p);
+    ERTS_UNREQ_PROC_MAIN_LOCK(c_p);
     flags = erts_call_trace(c_p, info, ms, reg, local, &tracer);
-    ERTS_SMP_REQ_PROC_MAIN_LOCK(c_p);
+    ERTS_REQ_PROC_MAIN_LOCK(c_p);
 
     /* restore cp after potential fixup */
     c_p->cp = cp_save;
@@ -1041,9 +1040,9 @@ do_call_trace(Process* c_p, ErtsCodeInfo* info, Eterm* reg,
 				  the funcinfo is above i. */
 	c_p->cp = (flags & MATCH_SET_EXCEPTION_TRACE) ?
 	    beam_exception_trace : beam_return_trace;
-	erts_smp_proc_lock(c_p, ERTS_PROC_LOCKS_ALL_MINOR);
+	erts_proc_lock(c_p, ERTS_PROC_LOCKS_ALL_MINOR);
 	ERTS_TRACE_FLAGS(c_p) |= F_EXCEPTION_TRACE;
-	erts_smp_proc_unlock(c_p, ERTS_PROC_LOCKS_ALL_MINOR);
+	erts_proc_unlock(c_p, ERTS_PROC_LOCKS_ALL_MINOR);
     } else
         c_p->stop = E;
     return tracer;
@@ -1060,7 +1059,7 @@ erts_trace_time_call(Process* c_p, ErtsCodeInfo *info, BpDataTime* bdt)
     Uint32 six = acquire_bp_sched_ix(c_p);
 
     ASSERT(c_p);
-    ASSERT(erts_smp_atomic32_read_acqb(&c_p->state) & (ERTS_PSFLG_RUNNING
+    ASSERT(erts_atomic32_read_acqb(&c_p->state) & (ERTS_PSFLG_RUNNING
 						       | ERTS_PSFLG_DIRTY_RUNNING));
 
     /* get previous timestamp and breakpoint
@@ -1141,7 +1140,7 @@ erts_trace_time_return(Process *p, ErtsCodeInfo *ci)
     Uint32 six = acquire_bp_sched_ix(p);
 
     ASSERT(p);
-    ASSERT(erts_smp_atomic32_read_acqb(&p->state) & (ERTS_PSFLG_RUNNING
+    ASSERT(erts_atomic32_read_acqb(&p->state) & (ERTS_PSFLG_RUNNING
 						     | ERTS_PSFLG_DIRTY_RUNNING));
 
     /* get previous timestamp and breakpoint
@@ -1223,7 +1222,7 @@ erts_is_mtrace_break(ErtsCodeInfo *ci, Binary **match_spec_ret,
 	    *match_spec_ret = bp->meta_ms;
 	}
 	if (tracer_ret) {
-            *tracer_ret = erts_smp_atomic_read_nob(&bp->meta_tracer->tracer);
+            *tracer_ret = erts_atomic_read_nob(&bp->meta_tracer->tracer);
 	}
 	return 1;
     }
@@ -1237,7 +1236,7 @@ erts_is_count_break(ErtsCodeInfo *ci, Uint *count_ret)
     
     if (bp) {
 	if (count_ret) {
-	    *count_ret = (Uint) erts_smp_atomic_read_nob(&bp->count->acount);
+	    *count_ret = (Uint) erts_atomic_read_nob(&bp->count->acount);
 	}
 	return 1;
     }
@@ -1318,7 +1317,7 @@ erts_find_local_func(ErtsCodeMFA *mfa) {
     n = (BeamInstr) code_hdr->num_functions;
     for (i = 0; i < n; ++i) {
 	ci = code_hdr->functions[i];
-	ASSERT(((BeamInstr) BeamOp(op_i_func_info_IaaI)) == ci->op);
+	ASSERT(BeamIsOpCode(ci->op, op_i_func_info_IaaI));
 	ASSERT(mfa->module == ci->mfa.module || is_nil(ci->mfa.module));
 	if (mfa->function == ci->mfa.function &&
 	    mfa->arity == ci->mfa.arity) {
@@ -1517,7 +1516,7 @@ set_function_break(ErtsCodeInfo *ci, Binary *match_spec, Uint break_flags,
     Uint common;
     ErtsBpIndex ix = erts_staging_bp_ix();
 
-    ERTS_SMP_LC_ASSERT(erts_has_code_write_permission());
+    ERTS_LC_ASSERT(erts_has_code_write_permission());
     g = ci->u.gen_bp;
     if (g == 0) {
 	int i;
@@ -1549,7 +1548,7 @@ set_function_break(ErtsCodeInfo *ci, Binary *match_spec, Uint break_flags,
 	    bp->flags &= ~ERTS_BPF_COUNT_ACTIVE;
 	} else {
 	    bp->flags |= ERTS_BPF_COUNT_ACTIVE;
-	    erts_smp_atomic_set_nob(&bp->count->acount, 0);
+	    erts_atomic_set_nob(&bp->count->acount, 0);
 	}
 	ASSERT((bp->flags & ~ERTS_BPF_ALL) == 0);
 	return;
@@ -1583,17 +1582,17 @@ set_function_break(ErtsCodeInfo *ci, Binary *match_spec, Uint break_flags,
 	MatchSetRef(match_spec);
 	bp->meta_ms = match_spec;
 	bmt = Alloc(sizeof(BpMetaTracer));
-	erts_smp_refc_init(&bmt->refc, 1);
+	erts_refc_init(&bmt->refc, 1);
         erts_tracer_update(&meta_tracer, tracer); /* copy tracer */
-	erts_smp_atomic_init_nob(&bmt->tracer, (erts_aint_t)meta_tracer);
+	erts_atomic_init_nob(&bmt->tracer, (erts_aint_t)meta_tracer);
 	bp->meta_tracer = bmt;
     } else if (break_flags & ERTS_BPF_COUNT) {
 	BpCount* bcp;
 
 	ASSERT((bp->flags & ERTS_BPF_COUNT) == 0);
 	bcp = Alloc(sizeof(BpCount));
-	erts_smp_refc_init(&bcp->refc, 1);
-	erts_smp_atomic_init_nob(&bcp->acount, 0);
+	erts_refc_init(&bcp->refc, 1);
+	erts_atomic_init_nob(&bcp->acount, 0);
 	bp->count = bcp;
     } else if (break_flags & ERTS_BPF_TIME_TRACE) {
 	BpDataTime* bdt;
@@ -1601,12 +1600,8 @@ set_function_break(ErtsCodeInfo *ci, Binary *match_spec, Uint break_flags,
 
 	ASSERT((bp->flags & ERTS_BPF_TIME_TRACE) == 0);
 	bdt = Alloc(sizeof(BpDataTime));
-	erts_smp_refc_init(&bdt->refc, 1);
-#ifdef ERTS_DIRTY_SCHEDULERS
+	erts_refc_init(&bdt->refc, 1);
 	bdt->n = erts_no_schedulers + 1;
-#else
-	bdt->n = erts_no_schedulers;
-#endif
 	bdt->hash = Alloc(sizeof(bp_time_hash_t)*(bdt->n));
 	for (i = 0; i < bdt->n; i++) {
 	    bp_hash_init(&(bdt->hash[i]), 32);
@@ -1638,7 +1633,7 @@ clear_function_break(ErtsCodeInfo *ci, Uint break_flags)
     Uint common;
     ErtsBpIndex ix = erts_staging_bp_ix();
 
-    ERTS_SMP_LC_ASSERT(erts_has_code_write_permission());
+    ERTS_LC_ASSERT(erts_has_code_write_permission());
 
     if ((g = ci->u.gen_bp) == NULL) {
 	return 1;
@@ -1671,8 +1666,8 @@ clear_function_break(ErtsCodeInfo *ci, Uint break_flags)
 static void
 bp_meta_unref(BpMetaTracer* bmt)
 {
-    if (erts_smp_refc_dectest(&bmt->refc, 0) <= 0) {
-        ErtsTracer trc = erts_smp_atomic_read_nob(&bmt->tracer);
+    if (erts_refc_dectest(&bmt->refc, 0) <= 0) {
+        ErtsTracer trc = erts_atomic_read_nob(&bmt->tracer);
         ERTS_TRACER_CLEAR(&trc);
 	Free(bmt);
     }
@@ -1681,7 +1676,7 @@ bp_meta_unref(BpMetaTracer* bmt)
 static void
 bp_count_unref(BpCount* bcp)
 {
-    if (erts_smp_refc_dectest(&bcp->refc, 0) <= 0) {
+    if (erts_refc_dectest(&bcp->refc, 0) <= 0) {
 	Free(bcp);
     }
 }
@@ -1689,7 +1684,7 @@ bp_count_unref(BpCount* bcp)
 static void
 bp_time_unref(BpDataTime* bdt)
 {
-    if (erts_smp_refc_dectest(&bdt->refc, 0) <= 0) {
+    if (erts_refc_dectest(&bdt->refc, 0) <= 0) {
 	Uint i = 0;
 	Uint j = 0;
 	Process *h_p = NULL;
@@ -1713,7 +1708,7 @@ bp_time_unref(BpDataTime* bdt)
 			    if (pbt) {
 				Free(pbt);
 			    }
-			    erts_smp_proc_unlock(h_p, ERTS_PROC_LOCK_MAIN);
+			    erts_proc_unlock(h_p, ERTS_PROC_LOCK_MAIN);
 			}
 		    }
 		}
@@ -1737,7 +1732,7 @@ check_break(ErtsCodeInfo *ci, Uint break_flags)
 {
     GenericBp* g = ci->u.gen_bp;
 
-    ASSERT(ci->op == (BeamInstr) BeamOp(op_i_func_info_IaaI));
+    ASSERT(BeamIsOpCode(ci->op, op_i_func_info_IaaI));
     if (erts_is_function_native(ci)) {
 	return 0;
     }
