@@ -47,6 +47,8 @@
 #define DFLAG_SEND_SENDER         0x80000
 #define DFLAG_BIG_SEQTRACE_LABELS 0x100000
 #define DFLAG_NO_MAGIC            0x200000 /* internal for pending connection */
+#define DFLAG_EXIT_PAYLOAD        0x400000
+#define DFLAG_FRAGMENTS           0x800000
 
 /* Mandatory flags for distribution */
 #define DFLAG_DIST_MANDATORY (DFLAG_EXTENDED_REFERENCES         \
@@ -75,7 +77,9 @@
                             | DFLAG_MAP_TAG                   \
                             | DFLAG_BIG_CREATION              \
                             | DFLAG_SEND_SENDER               \
-                            | DFLAG_BIG_SEQTRACE_LABELS)
+                            | DFLAG_BIG_SEQTRACE_LABELS       \
+                            | DFLAG_EXIT_PAYLOAD              \
+                            | DFLAG_FRAGMENTS)
 
 /* Flags addable by local distr implementations */
 #define DFLAG_DIST_ADDABLE    DFLAG_DIST_DEFAULT
@@ -99,26 +103,35 @@
                                | DFLAG_BIG_CREATION)
 
 /* opcodes used in distribution messages */
-#define DOP_LINK		1
-#define DOP_SEND		2
-#define DOP_EXIT		3
-#define DOP_UNLINK		4
+enum dop {
+    DOP_LINK                = 1,
+    DOP_SEND                = 2,
+    DOP_EXIT                = 3,
+    DOP_UNLINK              = 4,
 /* Ancient DOP_NODE_LINK (5) was here, can be reused */
-#define DOP_REG_SEND		6
-#define DOP_GROUP_LEADER	7
-#define DOP_EXIT2		8
+    DOP_REG_SEND            = 6,
+    DOP_GROUP_LEADER        = 7,
+    DOP_EXIT2               = 8,
 
-#define DOP_SEND_TT		12
-#define DOP_EXIT_TT		13
-#define DOP_REG_SEND_TT		16
-#define DOP_EXIT2_TT		18
+    DOP_SEND_TT             = 12,
+    DOP_EXIT_TT             = 13,
+    DOP_REG_SEND_TT         = 16,
+    DOP_EXIT2_TT            = 18,
 
-#define DOP_MONITOR_P		19
-#define DOP_DEMONITOR_P		20
-#define DOP_MONITOR_P_EXIT	21
+    DOP_MONITOR_P           = 19,
+    DOP_DEMONITOR_P         = 20,
+    DOP_MONITOR_P_EXIT      = 21,
 
-#define DOP_SEND_SENDER         22
-#define DOP_SEND_SENDER_TT      23
+    DOP_SEND_SENDER         = 22,
+    DOP_SEND_SENDER_TT      = 23,
+
+    /* These are used when DFLAG_EXIT_PAYLOAD is detected */
+    DOP_PAYLOAD_EXIT           = 24,
+    DOP_PAYLOAD_EXIT_TT        = 25,
+    DOP_PAYLOAD_EXIT2          = 26,
+    DOP_PAYLOAD_EXIT2_TT       = 27,
+    DOP_PAYLOAD_MONITOR_P_EXIT = 28
+};
 
 /* distribution trap functions */
 extern Export* dmonitor_node_trap;
@@ -129,15 +142,15 @@ typedef enum {
 } ErtsDSigPrepLock;
 
 
-typedef struct {
-    Process *proc;
-    DistEntry *dep;
-    Eterm node;   /* used if dep == NULL */
-    Eterm cid;
-    Eterm connection_id;
-    int no_suspend;
-    Uint32 flags;
-} ErtsDSigData;
+/* Must be larger or equal to 16 */
+#ifdef DEBUG
+#define ERTS_DIST_FRAGMENT_SIZE 16
+#else
+/* This should be made configurable */
+#define ERTS_DIST_FRAGMENT_SIZE (64 * 1024)
+#endif
+
+#define ERTS_DIST_FRAGMENT_HEADER_SIZE (1 + 1 + 8 + 8) /* magic, header, seq id, frag id*/
 
 #define ERTS_DE_BUSY_LIMIT (1024*1024)
 extern int erts_dist_buf_busy_limit;
@@ -159,123 +172,8 @@ extern int erts_is_alive;
 /* Pending connection; signals can be enqueued */
 #define ERTS_DSIG_PREP_PENDING	        4
 
-ERTS_GLB_INLINE int erts_dsig_prepare(ErtsDSigData *,
-				      DistEntry*,
-				      Process *,
-                                      ErtsProcLocks,
-				      ErtsDSigPrepLock,
-				      int,
-				      int);
-
-ERTS_GLB_INLINE
-void erts_schedule_dist_command(Port *, DistEntry *);
-
-int erts_auto_connect(DistEntry* dep, Process *proc, ErtsProcLocks proc_locks);
-
-#if ERTS_GLB_INLINE_INCL_FUNC_DEF
-
-ERTS_GLB_INLINE int 
-erts_dsig_prepare(ErtsDSigData *dsdp,
-		  DistEntry *dep,
-		  Process *proc,
-                  ErtsProcLocks proc_locks,
-		  ErtsDSigPrepLock dspl,
-		  int no_suspend,
-		  int connect)
-{
-    int res;
-
-    if (!erts_is_alive)
-	return ERTS_DSIG_PREP_NOT_ALIVE;
-    if (!dep) {
-        ASSERT(!connect);
-        return ERTS_DSIG_PREP_NOT_CONNECTED;
-    }
-
-#ifdef ERTS_ENABLE_LOCK_CHECK
-    if (connect) {
-        erts_proc_lc_might_unlock(proc, proc_locks);
-    }
-#endif
-
-retry:
-    erts_de_rlock(dep);
-
-    if (dep->state == ERTS_DE_STATE_CONNECTED) {
-	res = ERTS_DSIG_PREP_CONNECTED;
-    }
-    else if (dep->state == ERTS_DE_STATE_PENDING) {
-	res = ERTS_DSIG_PREP_PENDING;
-    }
-    else if (dep->state == ERTS_DE_STATE_EXITING) {
-	res = ERTS_DSIG_PREP_NOT_CONNECTED;
-	goto fail;
-    }
-    else if (connect) {
-        ASSERT(dep->state == ERTS_DE_STATE_IDLE);
-        erts_de_runlock(dep);
-        if (!erts_auto_connect(dep, proc, proc_locks)) {
-            return ERTS_DSIG_PREP_NOT_ALIVE;
-        }
-	goto retry;
-    }
-    else {
-        ASSERT(dep->state == ERTS_DE_STATE_IDLE);
-	res = ERTS_DSIG_PREP_NOT_CONNECTED;
-	goto fail;
-    }
-
-    if (no_suspend) {
-	if (erts_atomic32_read_acqb(&dep->qflgs) & ERTS_DE_QFLG_BUSY) {
-	    res = ERTS_DSIG_PREP_WOULD_SUSPEND;
-	    goto fail;
-	}
-    }
-    dsdp->proc = proc;
-    dsdp->dep = dep;
-    dsdp->cid = dep->cid;
-    dsdp->connection_id = dep->connection_id;
-    dsdp->no_suspend = no_suspend;
-    dsdp->flags = dep->flags;
-    if (dspl == ERTS_DSP_NO_LOCK)
-	erts_de_runlock(dep);
-    return res;
-
- fail:
-    erts_de_runlock(dep);
-    return res;
-}
-
-ERTS_GLB_INLINE
-void erts_schedule_dist_command(Port *prt, DistEntry *dist_entry)
-{
-    DistEntry *dep;
-    Eterm id;
-
-    if (prt) {
-	ERTS_LC_ASSERT(erts_lc_is_port_locked(prt));
-	ASSERT((erts_atomic32_read_nob(&prt->state)
-		& ERTS_PORT_SFLGS_DEAD) == 0);
-
-        dep = (DistEntry*) erts_prtsd_get(prt, ERTS_PRTSD_DIST_ENTRY);
-        ASSERT(dep);
-	id = prt->common.id;
-    }
-    else {
-	ASSERT(dist_entry);
-	ERTS_LC_ASSERT(erts_lc_rwmtx_is_rlocked(&dist_entry->rwmtx)
-			   || erts_lc_rwmtx_is_rwlocked(&dist_entry->rwmtx));
-	ASSERT(is_internal_port(dist_entry->cid));
-
- 	dep = dist_entry;
-	id = dep->cid;
-    }
-
-    if (!erts_atomic_xchg_mb(&dep->dist_cmd_scheduled, 1))
-	erts_port_task_schedule(id, &dep->dist_cmd, ERTS_PORT_TASK_DIST_CMD);
-}
-
-#endif
+/* dist_ctrl_{g,s}et_option/2 */
+#define ERTS_DIST_CTRL_OPT_GET_SIZE     ((Uint32) (1 << 0))
 
 #ifdef DEBUG
 #define ERTS_DBG_CHK_NO_DIST_LNK(D, R, L) \
@@ -336,41 +234,45 @@ enum erts_dsig_send_phase {
     ERTS_DSIG_SEND_PHASE_MSG_SIZE,
     ERTS_DSIG_SEND_PHASE_ALLOC,
     ERTS_DSIG_SEND_PHASE_MSG_ENCODE,
-    ERTS_DSIG_SEND_PHASE_FIN
+    ERTS_DSIG_SEND_PHASE_FIN,
+    ERTS_DSIG_SEND_PHASE_SEND
 };
 
-struct erts_dsig_send_context {
-    enum erts_dsig_send_phase phase;
-    Sint reds;
+typedef struct erts_dsig_send_context {
+    int connect;
+    int no_suspend;
+    int no_trap;
 
     Eterm ctl;
     Eterm msg;
-    int force_busy;
+    Eterm from;
+    Eterm ctl_heap[6];
+    Eterm return_term;
+
+    DistEntry *dep;
+    Eterm node;   /* used if dep == NULL */
+    Eterm cid;
+    Eterm connection_id;
+    int deref_dep;
+
+    enum erts_dsig_send_phase phase;
+    Sint reds;
+
     Uint32 max_finalize_prepend;
     Uint data_size, dhdr_ext_size;
     ErtsAtomCacheMap *acmp;
     ErtsDistOutputBuf *obuf;
+    Uint fragments;
     Uint32 flags;
     Process *c_p;
     union {
 	TTBSizeContext sc;
 	TTBEncodeContext ec;
     }u;
-};
 
-typedef struct {
-    int suspend;
-    int connect;
-
-    Eterm ctl_heap[6];
-    ErtsDSigData dsd;
-    DistEntry *dep;
-    int deref_dep;
-    struct erts_dsig_send_context dss;
-
-    Eterm return_term;
-}ErtsSendContext;
+} ErtsDSigSendContext;
 
+typedef struct dist_sequences DistSeqNode;
 
 /*
  * erts_dsig_send_* return values.
@@ -378,22 +280,23 @@ typedef struct {
 #define ERTS_DSIG_SEND_OK	0
 #define ERTS_DSIG_SEND_YIELD	1
 #define ERTS_DSIG_SEND_CONTINUE 2
+#define ERTS_DSIG_SEND_TOO_LRG  3
 
-extern int erts_dsig_send_link(ErtsDSigData *, Eterm, Eterm);
-extern int erts_dsig_send_msg(Eterm, Eterm, ErtsSendContext*);
-extern int erts_dsig_send_exit_tt(ErtsDSigData *, Eterm, Eterm, Eterm, Eterm);
-extern int erts_dsig_send_unlink(ErtsDSigData *, Eterm, Eterm);
-extern int erts_dsig_send_reg_msg(Eterm, Eterm, ErtsSendContext*);
-extern int erts_dsig_send_group_leader(ErtsDSigData *, Eterm, Eterm);
-extern int erts_dsig_send_exit(ErtsDSigData *, Eterm, Eterm, Eterm);
-extern int erts_dsig_send_exit2(ErtsDSigData *, Eterm, Eterm, Eterm);
-extern int erts_dsig_send_demonitor(ErtsDSigData *, Eterm, Eterm, Eterm, int);
-extern int erts_dsig_send_monitor(ErtsDSigData *, Eterm, Eterm, Eterm);
-extern int erts_dsig_send_m_exit(ErtsDSigData *, Eterm, Eterm, Eterm, Eterm);
+extern int erts_dsig_send_msg(ErtsDSigSendContext*, Eterm, Eterm);
+extern int erts_dsig_send_reg_msg(ErtsDSigSendContext*, Eterm, Eterm);
+extern int erts_dsig_send_link(ErtsDSigSendContext *, Eterm, Eterm);
+extern int erts_dsig_send_exit_tt(ErtsDSigSendContext *, Eterm, Eterm, Eterm, Eterm);
+extern int erts_dsig_send_unlink(ErtsDSigSendContext *, Eterm, Eterm);
+extern int erts_dsig_send_group_leader(ErtsDSigSendContext *, Eterm, Eterm);
+extern int erts_dsig_send_exit(ErtsDSigSendContext *, Eterm, Eterm, Eterm);
+extern int erts_dsig_send_exit2(ErtsDSigSendContext *, Eterm, Eterm, Eterm);
+extern int erts_dsig_send_demonitor(ErtsDSigSendContext *, Eterm, Eterm, Eterm);
+extern int erts_dsig_send_monitor(ErtsDSigSendContext *, Eterm, Eterm, Eterm);
+extern int erts_dsig_send_m_exit(ErtsDSigSendContext *, Eterm, Eterm, Eterm, Eterm);
 
-extern int erts_dsig_send(ErtsDSigData *dsdp, struct erts_dsig_send_context* ctx);
+extern int erts_dsig_send(ErtsDSigSendContext *dsdp);
 extern int erts_dsend_context_dtor(Binary*);
-extern Eterm erts_dsend_export_trap_context(Process* p, ErtsSendContext* ctx);
+extern Eterm erts_dsend_export_trap_context(Process* p, ErtsDSigSendContext* ctx);
 
 extern int erts_dist_command(Port *prt, int reds);
 extern void erts_dist_port_not_busy(Port *prt);
@@ -403,5 +306,19 @@ extern Uint erts_dist_cache_size(void);
 
 extern Sint erts_abort_connection_rwunlock(DistEntry *dep);
 
+extern void erts_dist_seq_tree_foreach(
+    DistEntry *dep,
+    int (*func)(ErtsDistExternal *, void*, Sint), void *args);
 
+extern int erts_dsig_prepare(ErtsDSigSendContext *,
+                             DistEntry*,
+                             Process *,
+                             ErtsProcLocks,
+                             ErtsDSigPrepLock,
+                             int,
+                             int,
+                             int);
+
+void erts_dist_print_procs_suspended_on_de(fmtfn_t to, void *to_arg);
+int erts_auto_connect(DistEntry* dep, Process *proc, ErtsProcLocks proc_locks);
 #endif
