@@ -532,10 +532,7 @@ re_compile(Process* p, Eterm arg1, Eterm arg2)
     int options = 0;
     int pflags = 0;
     int unicode = 0;
-#ifdef DEBUG
     int buffres;
-#endif
-
 
     if (parse_options(arg2,&options,NULL,&pflags,NULL,NULL,NULL,NULL)
 	< 0) {
@@ -556,12 +553,8 @@ re_compile(Process* p, Eterm arg1, Eterm arg2)
         BIF_ERROR(p,BADARG);
     }
     expr = erts_alloc(ERTS_ALC_T_RE_TMP_BUF, slen + 1);
-#ifdef DEBUG
-    buffres =
-#endif
-    erts_iolist_to_buf(arg1, expr, slen);
-
-    ASSERT(buffres >= 0);
+    buffres = erts_iolist_to_buf(arg1, expr, slen);
+    ASSERT(buffres >= 0); (void)buffres;
 
     expr[slen]='\0';
     result = erts_pcre_compile2(expr, options, &errcode, 
@@ -1052,9 +1045,7 @@ build_capture(Eterm capture_spec[CAPSPEC_SIZE], const pcre *code)
 			tmpb[ap->len] = '\0';
 		    } else {
 			ErlDrvSizeT slen;
-#ifdef DEBUG
 			int buffres;
-#endif
 
 			if (erts_iolist_size(val, &slen)) {
 			    goto error;
@@ -1068,11 +1059,8 @@ build_capture(Eterm capture_spec[CAPSPEC_SIZE], const pcre *code)
 			    }
 			}
 
-#ifdef DEBUG
-			buffres =
-#endif
-			erts_iolist_to_buf(val, tmpb, slen);
-			ASSERT(buffres >= 0);
+			buffres = erts_iolist_to_buf(val, tmpb, slen);
+			ASSERT(buffres >= 0); (void)buffres;
 			tmpb[slen] = '\0';
 		    }
 		    build_one_capture(code,&ri,&sallocated,has_dupnames,tmpb);
@@ -1145,9 +1133,7 @@ re_run(Process *p, Eterm arg1, Eterm arg2, Eterm arg3)
 	    const char *errstr = "";
 	    int errofset = 0;
 	    int capture_count;
-#ifdef DEBUG
 	    int buffres;
-#endif
 
 	    if (pflags & PARSE_FLAG_UNICODE && 
 		(!is_binary(arg2) || !is_binary(arg1) ||
@@ -1161,12 +1147,8 @@ re_run(Process *p, Eterm arg1, Eterm arg2, Eterm arg3)
 	    
 	    expr = erts_alloc(ERTS_ALC_T_RE_TMP_BUF, slen + 1);
 	    
-#ifdef DEBUG
-	    buffres =
-#endif
-	    erts_iolist_to_buf(arg2, expr, slen);
-
-	    ASSERT(buffres >= 0);
+	    buffres = erts_iolist_to_buf(arg2, expr, slen);
+	    ASSERT(buffres >= 0); (void)buffres;
 
 	    expr[slen]='\0';
 	    result = erts_pcre_compile2(expr, comp_options, &errcode, 
@@ -1317,9 +1299,7 @@ re_run(Process *p, Eterm arg1, Eterm arg2, Eterm arg3)
 	restart.subject = (char *) (pb->bytes+offset);
 	restart.flags |= RESTART_FLAG_SUBJECT_IN_BINARY;
     } else {
-#ifdef DEBUG
 	int buffres;
-#endif
 handle_iolist:
 	if (erts_iolist_size(arg1, &slength)) {
 	    erts_free(ERTS_ALC_T_RE_SUBJECT, restart.ovector);
@@ -1331,11 +1311,8 @@ handle_iolist:
 	}
 	restart.subject = erts_alloc(ERTS_ALC_T_RE_SUBJECT, slength);
 
-#ifdef DEBUG
-	buffres =
-#endif
-	erts_iolist_to_buf(arg1, restart.subject, slength);
-	ASSERT(buffres >= 0);
+	buffres = erts_iolist_to_buf(arg1, restart.subject, slength);
+	ASSERT(buffres >= 0); (void)buffres;
     }
 
     if (pflags & PARSE_FLAG_REPORT_ERRORS) {
@@ -1457,10 +1434,7 @@ re_inspect_2(BIF_ALIST_2)
     Eterm res;
     const pcre *code;
     byte *temp_alloc = NULL;
-#ifdef DEBUG
-    int infores;
-#endif
-    
+    int infores;    
 
     if (is_not_tuple(BIF_ARG_1) || (arityval(*tuple_val(BIF_ARG_1)) != 5)) {
 	goto error;
@@ -1484,12 +1458,8 @@ re_inspect_2(BIF_ALIST_2)
     if (erts_pcre_fullinfo(code, NULL, PCRE_INFO_OPTIONS, &options) != 0)
 	goto error;
 
-#ifdef DEBUG
-    infores =
-#endif
-    erts_pcre_fullinfo(code, NULL, PCRE_INFO_NAMECOUNT, &top);
-
-    ASSERT(infores == 0);
+    infores = erts_pcre_fullinfo(code, NULL, PCRE_INFO_NAMECOUNT, &top);
+    ASSERT(infores == 0); (void)infores;
 
     if (top <= 0) {
 	hp = HAlloc(BIF_P, 3);
@@ -1497,18 +1467,10 @@ re_inspect_2(BIF_ALIST_2)
 	erts_free_aligned_binary_bytes(temp_alloc);
 	BIF_RET(res);
     }
-#ifdef DEBUG
-    infores =
-#endif
-    erts_pcre_fullinfo(code, NULL, PCRE_INFO_NAMEENTRYSIZE, &entrysize);
-
+    infores = erts_pcre_fullinfo(code, NULL, PCRE_INFO_NAMEENTRYSIZE, &entrysize);
     ASSERT(infores == 0);
 
-#ifdef DEBUG
-    infores =
-#endif
-    erts_pcre_fullinfo(code, NULL, PCRE_INFO_NAMETABLE, &nametable);
-
+    infores = erts_pcre_fullinfo(code, NULL, PCRE_INFO_NAMETABLE, &nametable);
     ASSERT(infores == 0);
     
     has_dupnames = ((options & PCRE_DUPNAMES) != 0);
