@@ -41,16 +41,13 @@
 
 static IndexTable export_tables[ERTS_NUM_CODE_IX];  /* Active not locked */
 
-static erts_smp_atomic_t total_entries_bytes;
-
-#include "erl_smp.h"
+static erts_atomic_t total_entries_bytes;
 
 /* This lock protects the staging export table from concurrent access
  * AND it protects the staging table from becoming active.
  */
-erts_smp_mtx_t export_staging_lock;
+erts_mtx_t export_staging_lock;
 
-extern BeamInstr* em_call_error_handler;
 extern BeamInstr* em_call_traced_function;
 
 struct export_entry
@@ -85,17 +82,13 @@ static struct export_blob* entry_to_blob(struct export_entry* ee)
 void
 export_info(fmtfn_t to, void *to_arg)
 {
-#ifdef ERTS_SMP
     int lock = !ERTS_IS_CRASH_DUMPING;
     if (lock)
 	export_staging_lock();
-#endif
     index_info(to, to_arg, &export_tables[erts_active_code_ix()]);
     hash_info(to, to_arg, &export_tables[erts_staging_code_ix()].htable);
-#ifdef ERTS_SMP
     if (lock)
 	export_staging_unlock();
-#endif
 }
 
 
@@ -129,14 +122,17 @@ export_alloc(struct export_entry* tmpl_e)
 	Export* obj;
 
 	blob = (struct export_blob*) erts_alloc(ERTS_ALC_T_EXPORT, sizeof(*blob));
-	erts_smp_atomic_add_nob(&total_entries_bytes, sizeof(*blob));
+	erts_atomic_add_nob(&total_entries_bytes, sizeof(*blob));
 	obj = &blob->exp;
 	obj->info.op =  0;
 	obj->info.u.gen_bp = NULL;
 	obj->info.mfa.module = tmpl->info.mfa.module;
 	obj->info.mfa.function = tmpl->info.mfa.function;
 	obj->info.mfa.arity = tmpl->info.mfa.arity;
-	obj->beam[0] = (BeamInstr) em_call_error_handler;
+        obj->beam[0] = 0;
+        if (BeamOpsAreInitialized()) {
+            obj->beam[0] = BeamOpCodeAddr(op_call_error_handler);
+        }
 	obj->beam[1] = 0;
 
 	for (ix=0; ix<ERTS_NUM_CODE_IX; ix++) {
@@ -173,7 +169,7 @@ export_free(struct export_entry* obj)
     }
     DBG_TRACE_MFA_P(&blob->exp.info.mfa, "export blob deallocation at %p", &blob->exp);
     erts_free(ERTS_ALC_T_EXPORT, blob);
-    erts_smp_atomic_add_nob(&total_entries_bytes, -sizeof(*blob));
+    erts_atomic_add_nob(&total_entries_bytes, -sizeof(*blob));
 }
 
 void
@@ -182,9 +178,9 @@ init_export_table(void)
     HashFunctions f;
     int i;
 
-    erts_smp_mtx_init(&export_staging_lock, "export_tab", NIL,
+    erts_mtx_init(&export_staging_lock, "export_tab", NIL,
         ERTS_LOCK_FLAGS_PROPERTY_STATIC | ERTS_LOCK_FLAGS_CATEGORY_GENERIC);
-    erts_smp_atomic_init_nob(&total_entries_bytes, 0);
+    erts_atomic_init_nob(&total_entries_bytes, 0);
 
     f.hash = (H_FUN) export_hash;
     f.cmp  = (HCMP_FUN) export_cmp;
@@ -273,7 +269,7 @@ erts_find_function(Eterm m, Eterm f, unsigned int a, ErtsCodeIndex code_ix)
     ee = hash_get(&export_tables[code_ix].htable, init_template(&templ, m, f, a));
     if (ee == NULL ||
 	(ee->ep->addressv[code_ix] == ee->ep->beam &&
-	 ee->ep->beam[0] != (BeamInstr) BeamOp(op_i_generic_breakpoint))) {
+	 ! BeamIsOpCode(ee->ep->beam[0], op_i_generic_breakpoint))) {
 	return NULL;
     }
     return ee->ep;
@@ -373,7 +369,7 @@ int export_table_sz(void)
 }
 int export_entries_sz(void)
 {
-    return erts_smp_atomic_read_nob(&total_entries_bytes);
+    return erts_atomic_read_nob(&total_entries_bytes);
 }
 Export *export_get(Export *e)
 {
