@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 1996-2016. All Rights Reserved.
+ * Copyright Ericsson AB 1996-2018. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -45,7 +45,7 @@ typedef struct erl_module {
     int seen;			/* Used by finish_loading() */
 
     struct erl_module_instance curr;
-    struct erl_module_instance old; /* protected by "old_code" rwlock */
+    struct erl_module_instance old; /* active protected by "old_code" rwlock */
     struct erl_module_instance* on_load;
 } Module; 
 
@@ -72,29 +72,29 @@ int erts_is_old_code_rlocked(ErtsCodeIndex);
 
 #if ERTS_GLB_INLINE_INCL_FUNC_DEF
 
-extern erts_smp_rwmtx_t the_old_code_rwlocks[ERTS_NUM_CODE_IX];
+extern erts_rwmtx_t the_old_code_rwlocks[ERTS_NUM_CODE_IX];
 
 ERTS_GLB_INLINE void erts_rwlock_old_code(ErtsCodeIndex code_ix)
 {
-    erts_smp_rwmtx_rwlock(&the_old_code_rwlocks[code_ix]);
+    erts_rwmtx_rwlock(&the_old_code_rwlocks[code_ix]);
 }
 ERTS_GLB_INLINE void erts_rwunlock_old_code(ErtsCodeIndex code_ix)
 {
-    erts_smp_rwmtx_rwunlock(&the_old_code_rwlocks[code_ix]);
+    erts_rwmtx_rwunlock(&the_old_code_rwlocks[code_ix]);
 }
 ERTS_GLB_INLINE void erts_rlock_old_code(ErtsCodeIndex code_ix)
 {
-    erts_smp_rwmtx_rlock(&the_old_code_rwlocks[code_ix]);
+    erts_rwmtx_rlock(&the_old_code_rwlocks[code_ix]);
 }
 ERTS_GLB_INLINE void erts_runlock_old_code(ErtsCodeIndex code_ix)
 {
-    erts_smp_rwmtx_runlock(&the_old_code_rwlocks[code_ix]);
+    erts_rwmtx_runlock(&the_old_code_rwlocks[code_ix]);
 }
 
 #ifdef ERTS_ENABLE_LOCK_CHECK
 ERTS_GLB_INLINE int erts_is_old_code_rlocked(ErtsCodeIndex code_ix)
 {
-    return erts_smp_lc_rwmtx_is_rlocked(&the_old_code_rwlocks[code_ix]);
+    return erts_lc_rwmtx_is_rlocked(&the_old_code_rwlocks[code_ix]);
 }
 #endif
 
