@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1999-2017. All Rights Reserved.
+%% Copyright Ericsson AB 1999-2018. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -27,7 +27,8 @@
          constant_pools/1,constant_refc_binaries/1,
          fake_literals/1,
          false_dependency/1,coverage/1,fun_confusion/1,
-         t_copy_literals/1, t_copy_literals_frags/1]).
+         t_copy_literals/1, t_copy_literals_frags/1,
+         erl_544/1, max_heap_size/1]).
 
 -define(line_trace, 1).
 -include_lib("common_test/include/ct.hrl").
@@ -41,7 +42,8 @@ all() ->
      module_md5,
      constant_pools, constant_refc_binaries, fake_literals,
      false_dependency,
-     coverage, fun_confusion, t_copy_literals, t_copy_literals_frags].
+     coverage, fun_confusion, t_copy_literals, t_copy_literals_frags,
+     erl_544, max_heap_size].
 
 init_per_suite(Config) ->
     erts_debug:set_internal_state(available_internal_state, true),
@@ -330,6 +332,7 @@ constant_pools(Config) when is_list(Config) ->
     A = literals:a(),
     B = literals:b(),
     C = literals:huge_bignum(),
+    D = literals:funs(),
     process_flag(trap_exit, true),
     Self = self(),
 
@@ -343,7 +346,7 @@ constant_pools(Config) when is_list(Config) ->
     true = erlang:purge_module(literals),
     NoOldHeap ! done,
     receive
-        {'EXIT',NoOldHeap,{A,B,C}} ->
+        {'EXIT',NoOldHeap,{A,B,C,D}} ->
             ok;
         Other ->
             ct:fail({unexpected,Other})
@@ -360,7 +363,7 @@ constant_pools(Config) when is_list(Config) ->
     erlang:purge_module(literals),
     OldHeap ! done,
     receive
-	{'EXIT',OldHeap,{A,B,C,[1,2,3|_]=Seq}} when length(Seq) =:= 16 ->
+	{'EXIT',OldHeap,{A,B,C,D,[1,2,3|_]=Seq}} when length(Seq) =:= 16 ->
 	    ok
     end,
 
@@ -388,7 +391,7 @@ constant_pools(Config) when is_list(Config) ->
 	{'DOWN', Mon, process, Hib, Reason} ->
 	    {undef, [{no_module,
 		      no_function,
-		      [{A,B,C,[1,2,3|_]=Seq}], _}]} = Reason,
+		      [{A,B,C,D,[1,2,3|_]=Seq}], _}]} = Reason,
 	    16 = length(Seq)
     end,
     HeapSz = TotHeapSz, %% Ensure restored to hibernated state...
@@ -398,7 +401,9 @@ constant_pools(Config) when is_list(Config) ->
 no_old_heap(Parent) ->
     A = literals:a(),
     B = literals:b(),
-    Res = {A,B,literals:huge_bignum()},
+    C = literals:huge_bignum(),
+    D = literals:funs(),
+    Res = {A,B,C,D},
     Parent ! go,
     receive
         done ->
@@ -408,7 +413,9 @@ no_old_heap(Parent) ->
 old_heap(Parent) ->
     A = literals:a(),
     B = literals:b(),
-    Res = {A,B,literals:huge_bignum(),lists:seq(1, 16)},
+    C = literals:huge_bignum(),
+    D = literals:funs(),
+    Res = {A,B,C,D,lists:seq(1, 16)},
     create_old_heap(),
     Parent ! go,
     receive
@@ -419,7 +426,9 @@ old_heap(Parent) ->
 hibernated(Parent) ->
     A = literals:a(),
     B = literals:b(),
-    Res = {A,B,literals:huge_bignum(),lists:seq(1, 16)},
+    C = literals:huge_bignum(),
+    D = literals:funs(),
+    Res = {A,B,C,D,lists:seq(1, 16)},
     Parent ! go,
     erlang:hibernate(no_module, no_function, [Res]).
 
@@ -753,7 +762,8 @@ t_copy_literals_frags(Config) when is_list(Config) ->
                                           0, 1, 2, 3, 4, 5, 6, 7,
                                           8, 9,10,11,12,13,14,15,
                                           0, 1, 2, 3, 4, 5, 6, 7,
-                                          8, 9,10,11,12,13,14,15>>}]),
+                                          8, 9,10,11,12,13,14,15>>},
+                        {f, fun ?MODULE:all/0}]),
 
     {module, ?mod} = erlang:load_module(?mod, Bin),
     N = 6000,
@@ -794,6 +804,7 @@ literal_receiver() ->
             C = ?mod:c(),
             D = ?mod:d(),
             E = ?mod:e(),
+            F = ?mod:f(),
             literal_receiver();
         {Pid, sender_confirm} ->
             io:format("sender confirm ~w~n", [Pid]),
@@ -809,7 +820,8 @@ literal_sender(N, Recv) ->
                           ?mod:b(),
                           ?mod:c(),
                           ?mod:d(),
-                          ?mod:e()]},
+                          ?mod:e(),
+                          ?mod:f()]},
     literal_sender(N - 1, Recv).
 
 literal_switcher() ->
@@ -918,6 +930,86 @@ reloader(Mod,Code,Time) ->
               reloader(Mod,Code,Time)
     end.
 
+erl_544(Config) when is_list(Config) ->
+    case file:native_name_encoding() of
+        utf8 ->
+            {ok, CWD} = file:get_cwd(),
+            try
+                Mod = erl_544,
+                FileName = atom_to_list(Mod) ++ ".erl",
+                Priv = proplists:get_value(priv_dir, Config),
+                Data = proplists:get_value(data_dir, Config),
+                {ok, FileContent} = file:read_file(filename:join(Data,
+                                                                 FileName)),
+                Dir = filename:join(Priv, [16#2620,16#2620,16#2620]),
+                File = filename:join(Dir, FileName),
+                io:format("~ts~n", [File]),
+                ok = file:make_dir(Dir),
+                ok = file:set_cwd(Dir),
+                ok = file:write_file(File, [FileContent]),
+                {ok, Mod} = compile:file(File),
+                Res1 = (catch Mod:err()),
+                io:format("~p~n", [Res1]),
+                {'EXIT', {err, [{Mod, err, 0, Info1}|_]}} = Res1,
+                File = proplists:get_value(file, Info1),
+                Me = self(),
+                Go = make_ref(),
+                Tester = spawn_link(fun () ->
+                                            Mod:wait(Me, Go),
+                                            Mod:err()
+                                    end),
+                receive Go -> ok end,
+                Res2 = process_info(Tester, current_stacktrace),
+                io:format("~p~n", [Res2]),
+                {current_stacktrace, Stack} = Res2,
+                [{Mod, wait, 2, Info2}|_] = Stack,
+                File = proplists:get_value(file, Info2),
+                StackFun = fun(_, _, _) -> false end,
+                FormatFun = fun (Term, _) -> io_lib:format("~tp", [Term]) end,
+                Formated =
+                    erl_error:format_stacktrace(1, Stack, StackFun, FormatFun),
+                true = is_list(Formated),
+                ok
+            after
+                ok = file:set_cwd(CWD)
+            end,
+            ok;
+        _Enc ->
+            {skipped, "Only run when native file name encoding is utf8"}
+    end.
+
+%% Test that the copying of literals to a process during purging of
+%% literals will cause the process to be killed if the max heap size
+%% is exceeded.
+max_heap_size(_Config) ->
+    Mod = ?FUNCTION_NAME,
+    Value = [I || I <- lists:seq(1, 5000)],
+    Code = gen_lit(Mod, [{term,Value}]),
+    {module,Mod} = erlang:load_module(Mod, Code),
+    SpawnOpts = [monitor,
+                 {max_heap_size,
+                  #{size=>1024,
+                    kill=>true,
+                    error_logger=>true}}],
+    {Pid,Ref} = spawn_opt(fun() ->
+                                  max_heap_size_proc(Mod)
+                          end, SpawnOpts),
+    receive
+        {'DOWN',Ref,process,Pid,Reason} ->
+            killed = Reason;
+        Other ->
+            ct:fail({unexpected_message,Other})
+    after 10000 ->
+            ct:fail({process_did_not_die, Pid, erlang:process_info(Pid)})
+    end.
+
+max_heap_size_proc(Mod) ->
+    Value = Mod:term(),
+    code:delete(Mod),
+    code:purge(Mod),
+    receive
+        _ -> Value
+    end.
 
 %% Utilities.
 
