@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2007-2017. All Rights Reserved.
+ * Copyright Ericsson AB 2007-2018. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,16 +20,8 @@
 /*
  * Purpose: escript front-end.
  */
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
 
-#include "sys.h"
-#ifdef __WIN32__
-#include <winbase.h>
-#endif
-
-#include <ctype.h>
+#include "etc_common.h"
 
 static int debug = 0;		/* Bit flags for debug printouts. */
 
@@ -74,7 +66,6 @@ static void error(char* format, ...);
 static void* emalloc(size_t size);
 static void efree(void *p);
 static char* strsave(char* string);
-static void push_words(char* src);
 static int run_erlang(char* name, char** argv);
 static char* get_default_emulator(char* progname);
 #ifdef __WIN32__
@@ -147,15 +138,6 @@ get_env(char *key)
 #endif
 }
 
-static void
-free_env_val(char *value)
-{
-#ifdef __WIN32__
-    if (value)
-	efree(value);
-#endif
-}
-
 static void
 set_env(char *key, char *value)
 {
@@ -431,7 +413,6 @@ main(int argc, char** argv)
     int eargv_size;
     int eargc_base;		/* How many arguments in the base of eargv. */
     char* emulator;
-    char* env;
     char* basename;
     char* def_emu_lookup_path;
     char scriptname[PMAX];
@@ -513,7 +494,7 @@ main(int argc, char** argv)
     }
 
     /* Determine path to emulator */
-    emulator = env = get_env("ESCRIPT_EMULATOR");
+    emulator = get_env("ESCRIPT_EMULATOR");
 
     if (emulator == NULL) {
 	emulator = get_default_emulator(def_emu_lookup_path);
@@ -527,10 +508,9 @@ main(int argc, char** argv)
      */
 
     PUSH(emulator);
-    free_env_val(env);
 
     PUSH("+B");
-    PUSH2("-boot", "start_clean");
+    PUSH2("-boot", "no_dot_erlang");
     PUSH("-noshell");
 
     /*
@@ -583,26 +563,6 @@ main(int argc, char** argv)
     return run_erlang(eargv[0], eargv);
 }
 
-static void
-push_words(char* src)
-{
-    char sbuf[PMAX];
-    char* dst;
-
-    dst = sbuf;
-    while ((*dst++ = *src++) != '\0') {
-	if (isspace((int)*src)) {
-	    *dst = '\0';
-	    PUSH(strsave(sbuf));
-	    dst = sbuf;
-	    do {
-		src++;
-	    } while (isspace((int)*src));
-	}
-    }
-    if (sbuf[0])
-	PUSH(strsave(sbuf));
-}
 #ifdef __WIN32__
 wchar_t *make_commandline(char **argv)
 {
