@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1998-2017. All Rights Reserved.
+%% Copyright Ericsson AB 1998-2018. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -770,7 +770,7 @@ framework_call(Callback,Func,Args,DefaultReturn) ->
 			 end,
 	    case SetTcState of
 		true ->
-		    test_server:set_tc_state({framework,Mod,Func});
+		    test_server:set_tc_state({framework,{Mod,Func,Args}});
 		false ->
 		    ok
 	    end,
@@ -781,8 +781,8 @@ framework_call(Callback,Func,Args,DefaultReturn) ->
 	    catch
 		exit:Why ->
 		    EH(Why);
-		error:Why ->
-		    EH({Why,erlang:get_stacktrace()});
+		error:Why:Stacktrace ->
+		    EH({Why,Stacktrace});
 		throw:Why ->
 		    EH(Why)
 	    end;
