@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1997-2017. All Rights Reserved.
+%% Copyright Ericsson AB 1997-2018. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -66,7 +66,6 @@ suite() ->
      {timetrap,{minutes,10}}].
 
 all() -> 
-    test_lib:recompile(?MODULE),
     [self_compile_old_inliner,self_compile,
      {group,p}].
 
@@ -88,6 +87,7 @@ groups() ->
        string_table,otp_8949_a,split_cases]}].
 
 init_per_suite(Config) ->
+    test_lib:recompile(?MODULE),
     Config.
 
 end_per_suite(_Config) ->
@@ -170,7 +170,7 @@ try_it(Module, Conf) ->
 			atom_to_list(Module)),
     Out = proplists:get_value(priv_dir,Conf),
     io:format("Compiling: ~s\n", [Src]),
-    CompRc0 = compile:file(Src, [clint0,clint,{outdir,Out},report,
+    CompRc0 = compile:file(Src, [clint0,clint,ssalint,{outdir,Out},report,
 				 bin_opt_info|OtherOpts]),
     io:format("Result: ~p\n",[CompRc0]),
     {ok,_Mod} = CompRc0,
@@ -189,7 +189,7 @@ try_it(Module, Conf) ->
 
     ct:timetrap(Timetrap),
     io:format("Compiling (with old inliner): ~s\n", [Src]),
-    CompRc2 = compile:file(Src, [clint,
+    CompRc2 = compile:file(Src, [clint,ssalint,
 				 {outdir,Out},report,bin_opt_info,
 				 {inline,1000}|OtherOpts]),
     io:format("Result: ~p\n",[CompRc2]),
@@ -355,7 +355,7 @@ compile_compiler(Files, OutDir, Version, InlineOpts) ->
     io:format("~ts", [code:which(compile)]),
     io:format("Compiling ~s into ~ts", [Version,OutDir]),
     Opts = [report,
-	    clint0,clint,
+	    clint0,clint,ssalint,
 	    bin_opt_info,
 	    {outdir,OutDir},
 	    {d,'COMPILER_VSN',"\""++Version++"\""},
