@@ -42,13 +42,9 @@ groups() ->
 
 init_per_suite(Config) ->
     test_lib:recompile(?MODULE),
-    Pa = "-pa " ++ filename:dirname(code:which(?MODULE)),
-    {ok,Node} = start_node(compiler, Pa),
-    [{testing_node,Node}|Config].
+    Config.
 
-end_per_suite(Config) ->
-    Node = proplists:get_value(testing_node, Config),
-    test_server:stop_node(Node),
+end_per_suite(_Config) ->
     ok.
 
 init_per_group(_GroupName, Config) ->
@@ -89,26 +85,26 @@ attribute(Config) when is_list(Config) ->
 ?comp(maps_inline_test).
 
 try_inline(Mod, Config) ->
-    Node = proplists:get_value(testing_node, Config),
     Src = filename:join(proplists:get_value(data_dir, Config),
 			atom_to_list(Mod)),
     Out = proplists:get_value(priv_dir,Config),
 
     %% Normal compilation.
     io:format("Compiling: ~s\n", [Src]),
-    {ok,Mod} = compile:file(Src, [{outdir,Out},report,bin_opt_info,clint]),
+    {ok,Mod} = compile:file(Src, [{outdir,Out},report,
+                                  bin_opt_info,clint,ssalint]),
 
     ct:timetrap({minutes,10}),
-    NormalResult = rpc:call(Node, ?MODULE, load_and_call, [Out,Mod]),
+    NormalResult = load_and_call(Out, Mod),
 
     %% Inlining.
     io:format("Compiling with old inliner: ~s\n", [Src]),
     {ok,Mod} = compile:file(Src, [{outdir,Out},report,bin_opt_info,
-					{inline,1000},clint]),
+                                  {inline,1000},clint,ssalint]),
 
     %% Run inlined code.
     ct:timetrap({minutes,10}),
-    OldInlinedResult = rpc:call(Node, ?MODULE, load_and_call, [Out,Mod]),
+    OldInlinedResult = load_and_call(Out, Mod),
 
     %% Compare results.
     compare(NormalResult, OldInlinedResult),
@@ -117,11 +113,11 @@ try_inline(Mod, Config) ->
     %% Inlining.
     io:format("Compiling with new inliner: ~s\n", [Src]),
     {ok,Mod} = compile:file(Src, [{outdir,Out},report,
-					bin_opt_info,inline,clint]),
+					bin_opt_info,inline,clint,ssalint]),
 
     %% Run inlined code.
     ct:timetrap({minutes,10}),
-    InlinedResult = rpc:call(Node, ?MODULE, load_and_call, [Out,Mod]),
+    InlinedResult = load_and_call(Out, Mod),
 
     %% Compare results.
     compare(NormalResult, InlinedResult),
@@ -130,6 +126,11 @@ try_inline(Mod, Config) ->
     %% Delete Beam file.
     ok = file:delete(filename:join(Out, atom_to_list(Mod)++code:objfile_extension())),
 
+    %% Delete loaded module.
+    _ = code:purge(Mod),
+    _ = code:delete(Mod),
+    _ = code:purge(Mod),
+
     ok.
 
 compare(Same, Same) -> ok;
@@ -143,12 +144,6 @@ compare([H1|_], [H2|_]) ->
     ct:fail(different);
 compare([], []) -> ok.
 
-start_node(Name, Args) ->
-    case test_server:start_node(Name, slave, [{args,Args}]) of
-	{ok,Node} -> {ok, Node};
-	Error  -> ct:fail(Error)
-    end.
-
 load_and_call(Out, Module) ->
     io:format("Loading...\n",[]),
     code:purge(Module),
@@ -349,9 +344,8 @@ otp_7223_2({a}) ->
     1.
 
 coverage(Config) when is_list(Config) ->
-    Mod = bsdecode,
+    Mod = attribute,
     Src = filename:join(proplists:get_value(data_dir, Config), Mod),
-    {ok,Mod,_} = compile:file(Src, [binary,report,{inline,0},clint]),
-    {ok,Mod,_} = compile:file(Src, [binary,report,{inline,20},
-				    verbose,clint]),
+    {ok,Mod,_} = compile:file(Src, [binary,report,{inline,0},
+                                    clint,ssalint]),
     ok.
