@@ -70,7 +70,10 @@
 	t_bad_update/1,
 
         %% new in OTP 21
-        t_reused_key_variable/1
+        t_reused_key_variable/1,
+
+        %% new in OTP 22
+        t_mixed_clause/1,cover_beam_trim/1
     ]).
 
 suite() -> [].
@@ -124,7 +127,10 @@ all() ->
         t_bad_update,
 
         %% new in OTP 21
-        t_reused_key_variable
+        t_reused_key_variable,
+
+        %% new in OTP 22
+        t_mixed_clause,cover_beam_trim
     ].
 
 groups() -> [].
@@ -1373,22 +1379,22 @@ map_usage(Def, Used) ->
 
 
 t_guard_sequence(Config) when is_list(Config) ->
-	{1, "a"} = map_guard_sequence_1(#{seq=>1,val=>id("a")}),
-	{2, "b"} = map_guard_sequence_1(#{seq=>2,val=>id("b")}),
-	{3, "c"} = map_guard_sequence_1(#{seq=>3,val=>id("c")}),
-	{4, "d"} = map_guard_sequence_1(#{seq=>4,val=>id("d")}),
-	{5, "e"} = map_guard_sequence_1(#{seq=>5,val=>id("e")}),
+    {1, "a"} = map_guard_sequence_1(#{seq=>1,val=>id("a")}),
+    {2, "b"} = map_guard_sequence_1(#{seq=>2,val=>id("b")}),
+    {3, "c"} = map_guard_sequence_1(#{seq=>3,val=>id("c")}),
+    {4, "d"} = map_guard_sequence_1(#{seq=>4,val=>id("d")}),
+    {5, "e"} = map_guard_sequence_1(#{seq=>5,val=>id("e")}),
 
-	{1,M1}       = map_guard_sequence_2(M1 = id(#{a=>3})),
-	{2,M2}       = map_guard_sequence_2(M2 = id(#{a=>4, b=>4})),
-	{3,gg,M3}    = map_guard_sequence_2(M3 = id(#{a=>gg, b=>4})),
-	{4,sc,sc,M4} = map_guard_sequence_2(M4 = id(#{a=>sc, b=>3, c=>sc2})),
-	{5,kk,kk,M5} = map_guard_sequence_2(M5 = id(#{a=>kk, b=>other, c=>sc2})),
-	
-	%% error case
-	{'EXIT',{function_clause,_}} = (catch map_guard_sequence_1(#{seq=>6,val=>id("e")})),
-	{'EXIT',{function_clause,_}} = (catch map_guard_sequence_2(#{b=>5})),
-	ok.
+    {1,M1}       = map_guard_sequence_2(M1 = id(#{a=>3})),
+    {2,M2}       = map_guard_sequence_2(M2 = id(#{a=>4, b=>4})),
+    {3,gg,M3}    = map_guard_sequence_2(M3 = id(#{a=>gg, b=>4})),
+    {4,sc,sc,M4} = map_guard_sequence_2(M4 = id(#{a=>sc, b=>3, c=>sc2})),
+    {5,kk,kk,M5} = map_guard_sequence_2(M5 = id(#{a=>kk, b=>other, c=>sc2})),
+
+    %% error case
+    {'EXIT',{function_clause,_}} = (catch map_guard_sequence_1(#{seq=>6,val=>id("e")})),
+    {'EXIT',{function_clause,_}} = (catch map_guard_sequence_2(#{b=>5})),
+    ok.
 
 t_guard_sequence_large(Config) when is_list(Config) ->
     M0 = id(#{ 10=>a0,20=>b0,30=>"c0","40"=>"d0",<<"50">>=>"e0",{["00",03]}=>"10",
@@ -1443,21 +1449,21 @@ t_guard_sequence_large(Config) when is_list(Config) ->
                   18=>a8,28=>b8,38=>"c8","48"=>"d8",<<"58">>=>"e8",{["08"]}=>"18",
                   19=>a9,29=>b9,39=>"c9","49"=>"d9",<<"59">>=>"e9",{["09"]}=>"19" } => "large map key 2" }),
 
-	{1, "a"} = map_guard_sequence_1(M0#{seq=>1,val=>id("a")}),
-	{2, "b"} = map_guard_sequence_1(M0#{seq=>2,val=>id("b")}),
-	{3, "c"} = map_guard_sequence_1(M0#{seq=>3,val=>id("c")}),
-	{4, "d"} = map_guard_sequence_1(M0#{seq=>4,val=>id("d")}),
-	{5, "e"} = map_guard_sequence_1(M0#{seq=>5,val=>id("e")}),
+    {1, "a"} = map_guard_sequence_1(M0#{seq=>1,val=>id("a")}),
+    {2, "b"} = map_guard_sequence_1(M0#{seq=>2,val=>id("b")}),
+    {3, "c"} = map_guard_sequence_1(M0#{seq=>3,val=>id("c")}),
+    {4, "d"} = map_guard_sequence_1(M0#{seq=>4,val=>id("d")}),
+    {5, "e"} = map_guard_sequence_1(M0#{seq=>5,val=>id("e")}),
 
-	{1,M1}       = map_guard_sequence_2(M1 = id(M0#{a=>3})),
-	{2,M2}       = map_guard_sequence_2(M2 = id(M0#{a=>4, b=>4})),
-	{3,gg,M3}    = map_guard_sequence_2(M3 = id(M0#{a=>gg, b=>4})),
-	{4,sc,sc,M4} = map_guard_sequence_2(M4 = id(M0#{a=>sc, b=>3, c=>sc2})),
-	{5,kk,kk,M5} = map_guard_sequence_2(M5 = id(M0#{a=>kk, b=>other, c=>sc2})),
+    {1,M1}       = map_guard_sequence_2(M1 = id(M0#{a=>3})),
+    {2,M2}       = map_guard_sequence_2(M2 = id(M0#{a=>4, b=>4})),
+    {3,gg,M3}    = map_guard_sequence_2(M3 = id(M0#{a=>gg, b=>4})),
+    {4,sc,sc,M4} = map_guard_sequence_2(M4 = id(M0#{a=>sc, b=>3, c=>sc2})),
+    {5,kk,kk,M5} = map_guard_sequence_2(M5 = id(M0#{a=>kk, b=>other, c=>sc2})),
 
-	{'EXIT',{function_clause,_}} = (catch map_guard_sequence_1(M0#{seq=>6,val=>id("e")})),
-	{'EXIT',{function_clause,_}} = (catch map_guard_sequence_2(M0#{b=>5})),
-        ok.
+    {'EXIT',{function_clause,_}} = (catch map_guard_sequence_1(M0#{seq=>6,val=>id("e")})),
+    {'EXIT',{function_clause,_}} = (catch map_guard_sequence_2(M0#{b=>5})),
+    ok.
 
 map_guard_sequence_1(#{seq:=1=Seq, val:=Val}) -> {Seq,Val};
 map_guard_sequence_1(#{seq:=2=Seq, val:=Val}) -> {Seq,Val};
@@ -2079,7 +2085,7 @@ t_register_corruption(Config) when is_list(Config) ->
     {3,wanted,<<"value">>} = register_corruption_foo(wanted,M),
     ok.
 
-register_corruption_foo(A,#{a := V1, b := V2}) ->
+register_corruption_foo(_,#{a := V1, b := V2}) ->
     register_corruption_dummy_call(1,V1,V2);
 register_corruption_foo(A,#{b := V}) ->
     register_corruption_dummy_call(2,A,V);
@@ -2161,6 +2167,31 @@ t_reused_key_variable(Config) when is_list(Config) ->
             ok
     end.
 
+t_mixed_clause(_Config) ->
+    put(fool_inliner, x),
+    K = get(fool_inliner),
+    {42,100} = case #{K=>42,y=>100} of
+                   #{x:=X,y:=Y} ->
+                       {X,Y}
+               end,
+    nomatch = case #{K=>42,y=>100} of
+                  #{x:=X,y:=0} ->
+                      {X,Y};
+                  #{} ->
+                      nomatch
+              end,
+    ok.
+
+cover_beam_trim(_Config) ->
+    val = do_cover_beam_trim(id, max, max, id, #{id=>val}),
+    ok.
+
+do_cover_beam_trim(Id, OldMax, Max, Id, M) ->
+    OldMax = id(Max),
+    #{Id:=Val} = id(M),
+    Val.
+
+
 %% aux
 
 rand_terms(0) -> [];
