@@ -148,8 +148,21 @@ end_per_group(_, Config) ->
     end.
 
 %%--------------------------------------------------------------------
-init_per_testcase(_Case, Config) ->
-    Config.
+init_per_testcase(Case, Config) ->
+    case string:tokens(atom_to_list(Case),"_") of
+        ["sign","verify",Type|_] ->
+            skip_if_unsup(list_to_atom(Type), Config);
+
+        ["priv","encrypt","pub","decrypt",Type|_] ->
+            skip_if_unsup(list_to_atom(Type), Config);
+
+        ["get","pub","from","priv","key",Type|_] ->
+            skip_if_unsup(list_to_atom(Type), Config);
+
+        _ ->
+            Config
+    end.
+
 end_per_testcase(_Case, _Config) ->
     ok.
 
@@ -345,13 +358,13 @@ engine_list(Config) when is_list(Config) ->
             {skip, "OTP Test engine not found"};
         {ok, Engine} ->
             try
-                EngineList0 = crypto:engine_list(),
                 case crypto:engine_load(<<"dynamic">>,
                                         [{<<"SO_PATH">>, Engine},
                                          <<"LOAD">>],
                                         []) of
                     {ok, E} ->
                         EngineList0 = crypto:engine_list(),
+                        false = lists:member(<<"MD5">>, EngineList0),
                         ok = crypto:engine_add(E),
                         [<<"MD5">>] = lists:subtract(crypto:engine_list(), EngineList0),
                         ok = crypto:engine_remove(E),
@@ -674,7 +687,7 @@ ensure_load(Config) when is_list(Config) ->
     end.
 
 %%%----------------------------------------------------------------
-%%% Pub/priv key storage tests.  Thoose are for testing the crypto.erl
+%%% Pub/priv key storage tests.  Those are for testing the crypto.erl
 %%% support for using priv/pub keys stored in an engine.
 
 sign_verify_rsa(Config) ->
@@ -851,6 +864,19 @@ get_pub_from_priv_key_ecdsa(Config) ->
 %%%================================================================
 %%% Help for engine_stored_pub_priv_keys* test cases
 %%%
+skip_if_unsup(Type, Config) ->
+    case pkey_supported(Type) of
+        false ->
+            {skip, "Unsupported in this cryptolib"};
+        true ->
+            Config
+    end.
+
+
+pkey_supported(Type) ->
+    lists:member(Type, proplists:get_value(public_keys, crypto:supports(), [])).
+
+
 load_storage_engine(Config) ->
     load_storage_engine(Config, []).
 
