@@ -1,8 +1,4 @@
 %% -*- erlang-indent-level: 2 -*-
-%%-----------------------------------------------------------------------
-%% %CopyrightBegin%
-%%
-%% Copyright Ericsson AB 2006-2016. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -15,9 +11,6 @@
 %% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 %% See the License for the specific language governing permissions and
 %% limitations under the License.
-%%
-%% %CopyrightEnd%
-%%
 
 -module(dialyzer_cl_parse).
 
@@ -44,11 +37,12 @@ start() ->
   init(),
   Args = init:get_plain_arguments(),
   try
-    cl(Args)
+    Ret = cl(Args),
+    Ret
   catch
     throw:{dialyzer_cl_parse_error, Msg} -> {error, Msg};
-    _:R ->
-      Msg = io_lib:format("~p\n~p\n", [R, erlang:get_stacktrace()]),
+    _:R:S ->
+      Msg = io_lib:format("~tp\n~tp\n", [R, S]),
       {error, lists:flatten(Msg)}
   end.
 
@@ -88,7 +82,7 @@ cl(["--get_warnings"|T]) ->
 cl(["-D"|_]) ->
   cl_error("No defines specified after -D");
 cl(["-D"++Define|T]) ->
-  Def = re:split(Define, "=", [{return, list}]),
+  Def = re:split(Define, "=", [{return, list}, unicode]),
   append_defines(Def),
   cl(T);
 cl(["-h"|_]) ->
@@ -140,6 +134,9 @@ cl(["--raw"|T]) ->
 cl(["--fullpath"|T]) ->
   put(dialyzer_filename_opt, fullpath),
   cl(T);
+cl(["--no_indentation"|T]) ->
+  put(dialyzer_indent_opt, false),
+  cl(T);
 cl(["-pa", Path|T]) ->
   case code:add_patha(Path) of
     true -> cl(T);
@@ -260,6 +257,7 @@ init() ->
   put(dialyzer_options_files,     DefaultOpts#options.files),
   put(dialyzer_output_format,     formatted),
   put(dialyzer_filename_opt,      basename),
+  put(dialyzer_indent_opt,        ?INDENT_OPT),
   put(dialyzer_options_check_plt, DefaultOpts#options.check_plt),
   put(dialyzer_timing,            DefaultOpts#options.timing),
   put(dialyzer_solvers,           DefaultOpts#options.solvers),
@@ -301,6 +299,7 @@ cl_options() ->
    {output_file, get(dialyzer_output)},
    {output_format, get(dialyzer_output_format)},
    {filename_opt, get(dialyzer_filename_opt)},
+   {indent_opt, get(dialyzer_indent_opt)},
    {analysis_type, get(dialyzer_options_analysis_type)},
    {get_warnings, get(dialyzer_options_get_warnings)},
    {timing, get(dialyzer_timing)},
@@ -367,7 +366,7 @@ help_message() ->
 		[--build_plt] [--add_to_plt] [--remove_from_plt]
 		[--check_plt] [--no_check_plt] [--plt_info] [--get_warnings]
                 [--dump_callgraph file] [--no_native] [--fullpath]
-                [--statistics] [--no_native_cache]
+                [--no_indentation] [--statistics] [--no_native_cache]
 Options:
   files_or_dirs (for backwards compatibility also as: -c files_or_dirs)
       Use Dialyzer from the command line to detect defects in the
@@ -479,6 +478,9 @@ Options:
       caching.
   --fullpath
       Display the full path names of files for which warnings are emitted.
+  --no_indentation
+      Do not indent contracts and success typings. Note that this option has
+      no effect when combined with the --raw option.
   --gui
       Use the GUI.
 
