@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2010-2018. All Rights Reserved.
+%% Copyright Ericsson AB 2010-2019. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -539,8 +539,7 @@ add_transports(Config) ->
                         ++ [{unordered, unordered()} || T == sctp],
                         [{capabilities_cb, fun capx/2},
                          {pool_size, 8}
-                         | server_apps()]
-                        ++ [{spawn_opt, {erlang, spawn, []}} || CS]),
+                         | server_apps()]),
     Cs = [?util:connect(CN,
                         [T, {sender, CS} | client_opts(T)],
                         LRef,
