@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2006-2017. All Rights Reserved.
+%% Copyright Ericsson AB 2006-2018. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -61,19 +61,35 @@
 	 scheme_defaults/0, 
 	 encode/1, decode/1]).
 
--export_type([scheme/0, default_scheme_port_number/0]).
+-export_type([uri/0,
+              user_info/0,
+              scheme/0, default_scheme_port_number/0,
+              host/0,
+              path/0,
+              query/0,
+              fragment/0]).
 
+-type uri() :: string() | binary().
+-type user_info() :: string() | binary().
+-type scheme() :: atom().
+-type host() :: string() | binary().
+-type path() :: string() | binary().
+-type query() :: string() | binary().
+-type fragment() :: string() | binary().
+-type port_number() :: inet:port_number().
+-type default_scheme_port_number() :: port_number().
+-type hex_uri() :: string() | binary(). %% Hexadecimal encoded URI.
+-type maybe_hex_uri() :: string() | binary(). %% A possibly hexadecimal encoded URI.
+
+-type scheme_defaults() :: [{scheme(), default_scheme_port_number()}].
+-type scheme_validation_fun() :: fun((SchemeStr :: string() | binary()) ->
+                                            valid | {error, Reason :: term()}).
 
 %%%=========================================================================
 %%%  API
 %%%=========================================================================
 
--type scheme() :: atom().
--type default_scheme_port_number() :: pos_integer().
-
--spec scheme_defaults() ->
-    [{scheme(), default_scheme_port_number()}].
-
+-spec scheme_defaults() -> scheme_defaults().
 scheme_defaults() ->
     [{http,  80}, 
      {https, 443}, 
@@ -82,9 +98,20 @@ scheme_defaults() ->
      {sftp,  22}, 
      {tftp,  69}].
 
+-type parse_result() ::
+        {scheme(), user_info(), host(), port_number(), path(), query()} |
+        {scheme(), user_info(), host(), port_number(), path(), query(),
+         fragment()}.
+
+-spec parse(uri()) -> {ok, parse_result()} | {error, term()}.
 parse(AbsURI) ->
     parse(AbsURI, []).
 
+-spec parse(uri(), [Option]) -> {ok, parse_result()} | {error, term()} when
+      Option :: {ipv6_host_with_brackets, boolean()} |
+                {scheme_defaults, scheme_defaults()} |
+                {fragment, boolean()} |
+                {scheme_validation_fun, scheme_validation_fun() | none}.
 parse(AbsURI, Opts) ->
     case parse_scheme(AbsURI, Opts) of
 	{error, Reason} ->
@@ -105,6 +132,7 @@ reserved() ->
             $#, $[, $], $<, $>, $\", ${, $}, $|, %"
 			       $\\, $', $^, $%, $ ]).
 
+-spec encode(uri()) -> hex_uri().
 encode(URI) when is_list(URI) ->
     Reserved = reserved(), 
     lists:append([uri_encode(Char, Reserved) || Char <- URI]);
@@ -112,6 +140,7 @@ encode(URI) when is_binary(URI) ->
     Reserved = reserved(),
     << <<(uri_encode_binary(Char, Reserved))/binary>> || <<Char>> <= URI >>.
 
+-spec decode(maybe_hex_uri()) -> uri().
 decode(String) when is_list(String) ->
     do_decode(String);
 decode(String) when is_binary(String) ->
@@ -168,7 +197,7 @@ extract_scheme(Str, Opts) ->
 	{value, {scheme_validation_fun, Fun}} when is_function(Fun) ->
 	    case Fun(Str) of
 		valid ->
-		    {ok, list_to_atom(http_util:to_lower(Str))};
+		    {ok, to_atom(http_util:to_lower(Str))};
 		{error, Error} ->
 		    {error, Error}
 	    end;
