@@ -303,7 +303,10 @@ escaped_url_in_error_body(Config) when is_list(Config) ->
     %% Ask for a non-existing page(1)
     Path            = "/<b>this_is_bold<b>",
     HTMLEncodedPath = http_util:html_encode(Path),
-    URL2            = URL1 ++ Path,
+    URL2 = uri_string:recompose(#{scheme => "http",
+                                  host => "localhost",
+                                  port => Port,
+                                  path => Path}),
     {ok, {404, Body3}} = httpc:request(get, {URL2, []},
 				       [{url_encode,  true}, 
 					{version,     "HTTP/1.0"}],
