@@ -1,7 +1,7 @@
 %% 
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2005-2016. All Rights Reserved.
+%% Copyright Ericsson AB 2005-2019. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -358,22 +358,22 @@ run(Mod, Func, Args, Opts) ->
 	   "~n   StdM:        ~p",
 	   [M,Vsn,Dir,User,SecLevel,EngineID,CtxEngineID,Community,StdM]),
     case snmp_test_mgr:start([%% {agent, snmp_test_lib:hostname()},
-			      {packet_server_debug,true},
-			      {debug,true},
-			      {agent, get(master_host)}, 
-			      {ipfamily, get(ipfamily)},
-			      {agent_udp, 4000},
-			      {trap_udp, 5000},
-			      {recbuf,65535},
+			      {packet_server_debug, true},
+			      {debug,               true},
+			      {agent,               get(master_host)}, 
+			      {ipfamily,            get(ipfamily)},
+			      {agent_udp,           4000},
+			      {trap_udp,            5000},
+			      {recbuf,              65535},
 			      quiet,
 			      Vsn, 
-			      {community, Community},
-			      {user, User},
-			      {sec_level, SecLevel},
-			      {engine_id, EngineID},
-			      {context_engine_id, CtxEngineID},
-			      {dir, Dir},
-			      {mibs, mibs(StdM, M)}]) of
+			      {community,           Community},
+			      {user,                User},
+			      {sec_level,           SecLevel},
+			      {engine_id,           EngineID},
+			      {context_engine_id,   CtxEngineID},
+			      {dir,                 Dir},
+			      {mibs,                mibs(StdM, M)}]) of
 	{ok, _Pid} ->
 	    case (catch apply(Mod, Func, Args)) of
 		{'EXIT', Reason} ->
@@ -383,10 +383,18 @@ run(Mod, Func, Args, Opts) ->
 		    catch snmp_test_mgr:stop(),
 		    Res
 	    end;
-	Err ->
-	    io:format("Error starting manager: ~p\n", [Err]),
+
+	{error, Reason} ->
+	    ?EPRINT2("Failed starting (test) manager: "
+                     "~n   ~p", [Reason]),
 	    catch snmp_test_mgr:stop(),
-	    ?line ?FAIL({mgr_start, Err})
+	    ?line ?FAIL({mgr_start_error, Reason});
+
+	Err ->
+	    ?EPRINT2("Failed starting (test) manager: "
+                     "~n   ~p", [Err]),
+	    catch snmp_test_mgr:stop(),
+	    ?line ?FAIL({mgr_start_failure, Err})
     end.
 
 
@@ -445,6 +453,7 @@ start_agent(Config, Vsns, Opts) ->
 	    [{versions,         Vsns}, 
 	     {agent_type,       master},
 	     {agent_verbosity,  trace},
+             {get_mechanism,    snmp_agent_test_get},
 	     {db_dir,           AgentDbDir},
 	     {audit_trail_log,  [{type, read_write},
 				 {dir,  AgentLogDir},
@@ -463,20 +472,24 @@ start_agent(Config, Vsns, Opts) ->
 
     process_flag(trap_exit,true),
 
+    ?PRINT2("start_agent -> try start snmp app supervisor", []),
     {ok, AppSup} = snmp_app_sup:start_link(),
     unlink(AppSup),
     ?DBG("start_agent -> snmp app supervisor: ~p", [AppSup]),
 
-    ?DBG("start_agent -> start master agent",[]),
+    ?PRINT2("start_agent -> try start master agent",[]),
     ?line Sup = start_sup(Env), 
-
-    ?DBG("start_agent -> unlink from supervisor", []),
     ?line unlink(Sup),
+    ?DBG("start_agent -> snmp supervisor: ~p", [Sup]),
+
+    ?PRINT2("start_agent -> try (rpc) start sub agent on ~p", [SaNode]),
     ?line SaDir = ?config(sa_dir, Config),
-    ?DBG("start_agent -> (rpc) start sub on ~p", [SaNode]),
     ?line {ok, Sub} = start_sub_sup(SaNode, SaDir),
-    ?DBG("start_agent -> done",[]),
-    ?line [{snmp_sup, {Sup, self()}}, {snmp_sub, Sub} | Config].
+    ?DBG("start_agent -> done", []),
+
+    ?line [{snmp_app_sup, AppSup}, 
+           {snmp_sup,     {Sup, self()}}, 
+           {snmp_sub,     Sub} | Config].
 
 
 app_agent_env_init(Env0, Opts) ->
@@ -669,35 +682,52 @@ merge_agent_options([{Key, _Value} = Opt|Opts], Options) ->
 
 
 stop_agent(Config) when is_list(Config) ->
-    ?LOG("stop_agent -> entry with"
-	 "~n   Config: ~p",[Config]),
+    ?PRINT2("stop_agent -> entry with"
+            "~n   Config: ~p",[Config]),
 
-    {Sup, Par} = ?config(snmp_sup, Config),
-    ?DBG("stop_agent -> attempt to stop (sup) ~p"
-	"~n   Sup: ~p"
-	"~n   Par: ~p",
-	[Sup, 
-	(catch process_info(Sup)),
-	(catch process_info(Par))]),
-    
-    _Info = agent_info(Sup),
-    ?DBG("stop_agent -> Agent info: "
-	 "~n   ~p", [_Info]),
-    
-    stop_sup(Sup, Par),
 
-    {Sup2, Par2} = ?config(snmp_sub, Config),
-    ?DBG("stop_agent -> attempt to stop (sub) ~p"
-	"~n   Sup2: ~p"
-	"~n   Par2: ~p",
-	[Sup2,
-	(catch process_info(Sup2)),
-	(catch process_info(Par2))]),
-    stop_sup(Sup2, Par2),
+    %% Stop the sub-agent (the agent supervisor)
+    {SubSup, SubPar} = ?config(snmp_sub, Config),
+    ?PRINT2("stop_agent -> attempt to stop sub agent (~p)"
+            "~n   Sub Sup info: "
+            "~n      ~p"
+            "~n   Sub Par info: "
+            "~n      ~p",
+            [SubSup,
+             (catch process_info(SubSup)),
+             (catch process_info(SubPar))]),
+    stop_sup(SubSup, SubPar),
+    Config2 = lists:keydelete(snmp_sub, 1, Config),
 
-    ?DBG("stop_agent -> done - now cleanup config", []),
-    C1 = lists:keydelete(snmp_sup, 1, Config),
-    lists:keydelete(snmp_sub, 1, C1).
+
+    %% Stop the master-agent (the top agent supervisor)
+    {MasterSup, MasterPar} = ?config(snmp_sup, Config),
+    ?PRINT2("stop_agent -> attempt to stop master agent (~p)"
+            "~n   Master Sup: "
+            "~n      ~p"
+            "~n   Master Par: "
+            "~n      ~p"
+            "~n   Agent Info: "
+            "~n      ~p",
+            [MasterSup, 
+             (catch process_info(MasterSup)),
+             (catch process_info(MasterPar)),
+             agent_info(MasterSup)]),
+    stop_sup(MasterSup, MasterPar),
+    Config3 = lists:keydelete(snmp_sup, 1, Config2),
+
+
+    %% Stop the top supervisor (of the snmp app)
+    AppSup = ?config(snmp_app_sup, Config),
+    ?PRINT2("stop_agent -> attempt to app sup ~p"
+            "~n   App Sup: ~p",
+            [AppSup, 
+             (catch process_info(AppSup))]),
+    Config4 = lists:keydelete(snmp_app_sup, 1, Config3),
+
+
+    ?PRINT2("stop_agent -> done", []),
+    Config4.
 
 
 start_sup(Env) ->
@@ -727,7 +757,6 @@ stop_sup(Pid, _) ->
     ?LOG("stop_sup -> attempt to stop ~p", [Pid]),
     Ref = erlang:monitor(process, Pid),
     ?LOG("stop_sup -> Ref: ~p", [Ref]),
-    %% Pid ! {'EXIT', Parent, shutdown}, % usch
     exit(Pid, kill), 
     await_stopped(Pid, Ref).
 
@@ -863,13 +892,15 @@ expect(Mod, Line, Type, Enterp, Generic, Specific, ExpVBs) ->
     expect2(Mod, Line, Fun).
 
 expect2(Mod, Line, F) ->
-    io:format("EXPECT for ~w:~w~n", [Mod, Line]),
+    io_format_expect("for ~w:~w", [Mod, Line]),
     case F() of
 	{error, Reason} ->
-	    io:format("EXPECT failed at ~w:~w => ~n~p~n", [Mod, Line, Reason]),
+	    io_format_expect("failed at ~w:~w => "
+                             "~n      ~p", [Mod, Line, Reason]),
 	    throw({error, {expect, Mod, Line, Reason}});
 	Else ->
-	    io:format("EXPECT result for ~w:~w => ~n~p~n", [Mod, Line, Else]),
+	    io_format_expect("result for ~w:~w => "
+                             "~n      ~p", [Mod, Line, Else]),
 	    Else
     end.
 
@@ -898,20 +929,27 @@ receive_trap(To) ->
     end.
 
 
+io_format_expect(F) ->
+    io_format_expect(F, []).
+
+io_format_expect(F, A) ->
+    ?PRINT2("EXPECT " ++ F, A).
+    
+
 do_expect(Expect) when is_atom(Expect) ->
     do_expect({Expect, get_timeout()});
 
 do_expect({any_pdu, To}) 
   when is_integer(To) orelse (To =:= infinity) ->
-    io:format("EXPECT any PDU~n", []),
+    io_format_expect("any PDU"),
     receive_pdu(To);
 
 do_expect({any_trap, To}) ->
-    io:format("EXPECT any TRAP within ~w~n", [To]),
+    io_format_expect("any TRAP within ~w", [To]),
     receive_trap(To);
 
 do_expect({timeout, To}) ->
-    io:format("EXPECT nothing within ~w~n", [To]),
+    io_format_expect("nothing within ~w", [To]),
     receive
 	X ->
 	    {error, {unexpected, X}}
@@ -923,16 +961,16 @@ do_expect({timeout, To}) ->
 do_expect({Err, To}) 
   when (is_atom(Err) andalso 
 	((is_integer(To) andalso To > 0) orelse (To =:= infinity))) ->
-    io:format("EXPECT error ~w within ~w~n", [Err, To]),
+    io_format_expect("error ~w within ~w", [Err, To]),
     do_expect({{error, Err}, To});
 
 do_expect({error, Err}) when is_atom(Err) ->
     Check = fun(_, R) -> R end,
-    io:format("EXPECT error ~w~n", [Err]),
+    io_format_expect("error ~w", [Err]),
     do_expect2(Check, any, Err, any, any, get_timeout());
 do_expect({{error, Err}, To}) ->
     Check = fun(_, R) -> R end,
-    io:format("EXPECT error ~w within ~w~n", [Err, To]),
+    io_format_expect("error ~w within ~w", [Err, To]),
     do_expect2(Check, any, Err, any, any, To);
 
 %% exp_varbinds() -> [exp_varbind()]
@@ -942,25 +980,23 @@ do_expect({{error, Err}, To}) ->
 %% ExpVBs         -> exp_varbinds() | {VbsCondition, exp_varbinds()}
 do_expect(ExpVBs) ->
     Check = fun(_, R) -> R end,
-    io:format("EXPECT 'get-response'"
-	      "~n   with"
-	      "~n      Varbinds: ~p~n", [ExpVBs]),
+    io_format_expect("'get-response'"
+                     "~n   with"
+                     "~n      Varbinds: ~p", [ExpVBs]),
     do_expect2(Check, 'get-response', noError, 0, ExpVBs, get_timeout()).
 
 
 do_expect(v2trap, ExpVBs) ->
     Check = fun(_, R) -> R end,
-    io:format("EXPECT 'snmpv2-trap'"
-	      "~n   with"
-	      "~n      Varbinds: ~p~n", [ExpVBs]),
+    io_format_expect("'snmpv2-trap' with"
+                     "~n      Varbinds: ~p", [ExpVBs]),
     do_expect2(Check, 'snmpv2-trap', noError, 0, ExpVBs, get_timeout());
 
 
 do_expect(report, ExpVBs) ->
     Check = fun(_, R) -> R end,
-    io:format("EXPECT 'report'"
-	      "~n   with"
-	      "~n      Varbinds: ~p~n", [ExpVBs]),
+    io_format_expect("'report' with"
+                     "~n      Varbinds: ~p", [ExpVBs]),
     do_expect2(Check, 'report', noError, 0, ExpVBs, get_timeout());
 
 
@@ -969,9 +1005,8 @@ do_expect(inform, ExpVBs) ->
 
 do_expect({inform, false}, ExpVBs) ->
     Check = fun(_, R) -> R end,
-    io:format("EXPECT 'inform-request' (false)"
-	      "~n   with"
-	      "~n      Varbinds: ~p~n", [ExpVBs]),
+    io_format_expect("'inform-request' (false) with"
+                     "~n      Varbinds: ~p", [ExpVBs]),
     do_expect2(Check, 'inform-request', noError, 0, ExpVBs, get_timeout());
 
 do_expect({inform, true}, ExpVBs) ->
@@ -985,9 +1020,8 @@ do_expect({inform, true}, ExpVBs) ->
 	   (_, Err) ->
 		Err
 	end,
-    io:format("EXPECT 'inform-request' (true)"
-	      "~n   with"
-	      "~n      Varbinds: ~p~n", [ExpVBs]),
+    io_format_expect("'inform-request' (true) with"
+                     "~n      Varbinds: ~p", [ExpVBs]),
     do_expect2(Check, 'inform-request', noError, 0, ExpVBs, get_timeout());
 
 do_expect({inform, {error, EStat, EIdx}}, ExpVBs) 
@@ -1002,11 +1036,10 @@ do_expect({inform, {error, EStat, EIdx}}, ExpVBs)
 	   (_, Err) ->
 		Err
 	end,
-    io:format("EXPECT 'inform-request' (error)"
-	      "~n   with"
-	      "~n      Error Status: ~p"
-	      "~n      Error Index:  ~p"
-	      "~n      Varbinds:     ~p~n", [EStat, EIdx, ExpVBs]),
+    io_format_expect("'inform-request' (error) with"
+                     "~n      Error Status: ~p"
+                     "~n      Error Index:  ~p"
+                     "~n      Varbinds:     ~p", [EStat, EIdx, ExpVBs]),
     do_expect2(Check, 'inform-request', noError, 0, ExpVBs, get_timeout()).
 
 
@@ -1017,26 +1050,23 @@ do_expect(Err, Idx, ExpVBs, To)
   when is_atom(Err) andalso 
        (is_integer(Idx) orelse is_list(Idx) orelse (Idx == any)) ->
     Check = fun(_, R) -> R end,
-    io:format("EXPECT 'get-response'"
-	      "~n   with"
-	      "~n      Error:    ~p"
-	      "~n      Index:    ~p"
-	      "~n      Varbinds: ~p"
-	      "~n   within ~w~n", [Err, Idx, ExpVBs, To]),
+    io_format_expect("'get-response' withing ~w ms with"
+                     "~n      Error:    ~p"
+                     "~n      Index:    ~p"
+                     "~n      Varbinds: ~p", [To, Err, Idx, ExpVBs]),
     do_expect2(Check, 'get-response', Err, Idx, ExpVBs, To).
 
 
 do_expect(Type, Enterp, Generic, Specific, ExpVBs) ->
-    do_expect(Type, Enterp, Generic, Specific, ExpVBs, 3500).
+    do_expect(Type, Enterp, Generic, Specific, ExpVBs, get_timeout()).
 
 do_expect(trap, Enterp, Generic, Specific, ExpVBs, To) ->
-    io:format("EXPECT trap"
-	      "~n   with"
-	      "~n      Enterp:   ~w"
-	      "~n      Generic:  ~w"
-	      "~n      Specific: ~w"
-	      "~n      Varbinds: ~w"
-	      "~n   within ~w~n", [Enterp, Generic, Specific, ExpVBs, To]),
+    io_format_expect("trap within ~w ms with"
+                     "~n      Enterp:   ~w"
+                     "~n      Generic:  ~w"
+                     "~n      Specific: ~w"
+                     "~n      Varbinds: ~w",
+                     [To, Enterp, Generic, Specific, ExpVBs]),
     PureE = purify_oid(Enterp),
     case receive_trap(To) of
 	#trappdu{enterprise    = PureE, 
@@ -1071,46 +1101,46 @@ do_expect2(Check, Type, Err, Idx, ExpVBs, To)
 	#pdu{type         = Type, 
 	     error_status = Err,
 	     error_index  = Idx} when ExpVBs =:= any -> 
-	    io:format("EXPECT received expected pdu (1)~n", []),
+	    io_format_expect("received expected pdu (1)"),
 	    ok;
 
 	#pdu{type         = Type, 
 	     request_id   = ReqId, 
 	     error_status = Err2,
 	     error_index  = Idx} when ExpVBs =:= any -> 
-	    io:format("EXPECT received expected pdu with "
-		      "unexpected error status (2): "
-		      "~n   Error Status: ~p~n", [Err2]),
+	    io_format_expect("received expected pdu with "
+                             "unexpected error status (2): "
+                             "~n   Error Status: ~p", [Err2]),
 	    {error, {unexpected_error_status, Err, Err2, ReqId}};
 
 	#pdu{error_status = Err} when (Type   =:= any) andalso 
 				      (Idx    =:= any) andalso 
 				      (ExpVBs =:= any) -> 
-	    io:format("EXPECT received expected pdu (3)~n", []),
+	    io_format_expect("received expected pdu (3)"),
 	    ok;
 
 	#pdu{request_id   = ReqId, 
 	     error_status = Err2} when (Type   =:= any) andalso 
 				       (Idx    =:= any) andalso 
 				       (ExpVBs =:= any) -> 
-	    io:format("EXPECT received expected pdu with "
-		      "unexpected error status (4): "
-		      "~n   Error Status: ~p~n", [Err2]),
+	    io_format_expect("received expected pdu with "
+                             "unexpected error status (4): "
+                             "~n   Error Status: ~p", [Err2]),
 	    {error, {unexpected_error_status, Err, Err2, ReqId}};
 
 	#pdu{type         = Type, 
 	     error_status = Err} when (Idx =:= any) andalso 
 				      (ExpVBs =:= any) -> 
-	    io:format("EXPECT received expected pdu (5)~n", []),
+	    io_format_expect("received expected pdu (5)", []),
 	    ok;
 
 	#pdu{type         = Type, 
 	     request_id   = ReqId, 
 	     error_status = Err2} when (Idx =:= any) andalso 
 				       (ExpVBs =:= any) -> 
-	    io:format("EXPECT received expected pdu with "
-		      "unexpected error status (6): "
-		      "~n   Error Status: ~p~n", [Err2]),
+	    io_format_expect("received expected pdu with "
+                             "unexpected error status (6): "
+                             "~n   Error Status: ~p", [Err2]),
 	    {error, {unexpected_error_status, Err, Err2, ReqId}};
 
 	#pdu{type         = Type, 
@@ -1119,13 +1149,13 @@ do_expect2(Check, Type, Err, Idx, ExpVBs, To)
 	     error_index  = EI} when is_list(Idx) andalso (ExpVBs =:= any) -> 
 	    case lists:member(EI, Idx) of
 		true ->
-		    io:format("EXPECT received expected pdu with "
-			      "expected error index (7)~n", []),
+		    io_format_expect("received expected pdu with "
+                                     "expected error index (7)"),
 		    ok;
 		false ->
-		    io:format("EXPECT received expected pdu with "
-			      "unexpected error index (8): "
-			      "~n   Error Index: ~p~n", [EI]),
+		    io_format_expect("received expected pdu with "
+                                     "unexpected error index (8): "
+                                     "~n   Error Index: ~p", [EI]),
 		    {error, {unexpected_error_index, EI, Idx, ReqId}}
 	    end;
 
@@ -1135,15 +1165,15 @@ do_expect2(Check, Type, Err, Idx, ExpVBs, To)
 	     error_index  = EI} when is_list(Idx) andalso (ExpVBs =:= any) -> 
 	    case lists:member(EI, Idx) of
 		true ->
-		    io:format("EXPECT received expected pdu with "
-			      "unexpected error status (9): "
-			      "~n   Error Status: ~p~n", [Err2]),
+		    io_format_expect("received expected pdu with "
+                                     "unexpected error status (9): "
+                                     "~n   Error Status: ~p", [Err2]),
 		    {error, {unexpected_error_status, Err, Err2, ReqId}};
 		false ->
-		    io:format("EXPECT received expected pdu with "
-			      "unexpected error (10): "
-			      "~n   Error Status: ~p"
-			      "~n   Error index:  ~p~n", [Err2, EI]),
+		    io_format_expect("received expected pdu with "
+                                     "unexpected error (10): "
+                                     "~n   Error Status: ~p"
+                                     "~n   Error index:  ~p", [Err2, EI]),
 		    {error, {unexpected_error, {Err, Idx}, {Err2, EI}, ReqId}}
 	    end;
 
@@ -1151,12 +1181,12 @@ do_expect2(Check, Type, Err, Idx, ExpVBs, To)
 	     request_id   = ReqId, 
 	     error_status = Err2, 
 	     error_index  = Idx2} when ExpVBs =:= any ->
-	    io:format("EXPECT received unexpected pdu with (11) "
-		      "~n   Type:         ~p"
-		      "~n   ReqId:        ~p"
-		      "~n   Errot status: ~p"
-		      "~n   Error index:  ~p"
-		      "~n", [Type2, ReqId, Err2, Idx2]),
+	    io_format_expect("received unexpected pdu with (11) "
+                             "~n   Type:         ~p"
+                             "~n   ReqId:        ~p"
+                             "~n   Errot status: ~p"
+                             "~n   Error index:  ~p",
+                             [Type2, ReqId, Err2, Idx2]),
 	    {error, 
 	     {unexpected_pdu, 
 	      {Type, Err, Idx}, {Type2, Err2, Idx2}, ReqId}};
@@ -1165,26 +1195,26 @@ do_expect2(Check, Type, Err, Idx, ExpVBs, To)
 	     error_status = Err, 
 	     error_index  = Idx,
 	     varbinds     = VBs} = PDU ->
-	    io:format("EXPECT received pdu (12): "
-		      "~n   [exp] Type:         ~p"
-		      "~n   [exp] Error Status: ~p"
-		      "~n   [exp] Error Index:  ~p"
-		      "~n   VBs:                ~p"
-		      "~nwhen"
-		      "~n   ExpVBs:             ~p"
-		      "~n", [Type, Err, Idx, VBs, ExpVBs]),
+	    io_format_expect("received pdu (12): "
+                             "~n   [exp] Type:         ~p"
+                             "~n   [exp] Error Status: ~p"
+                             "~n   [exp] Error Index:  ~p"
+                             "~n   VBs:                ~p"
+                             "~nwhen"
+                             "~n   ExpVBs:             ~p",
+                             [Type, Err, Idx, VBs, ExpVBs]),
 	    Check(PDU, check_vbs(purify_oids(ExpVBs), VBs));
 
 	#pdu{type         = Type, 
 	     error_status = Err, 
 	     varbinds     = VBs} = PDU when Idx =:= any ->
-	    io:format("EXPECT received pdu (13): "
-		      "~n   [exp] Type:         ~p"
-		      "~n   [exp] Error Status: ~p"
-		      "~n   VBs:                ~p"
-		      "~nwhen"
-		      "~n   ExpVBs:             ~p"
-		      "~n", [Type, Err, VBs, ExpVBs]),
+	    io_format_expect("received pdu (13): "
+                             "~n   [exp] Type:         ~p"
+                             "~n   [exp] Error Status: ~p"
+                             "~n   VBs:                ~p"
+                             "~nwhen"
+                             "~n   ExpVBs:             ~p",
+                             [Type, Err, VBs, ExpVBs]),
 	    Check(PDU, check_vbs(purify_oids(ExpVBs), VBs));
 
 	#pdu{type         = Type, 
@@ -1192,15 +1222,15 @@ do_expect2(Check, Type, Err, Idx, ExpVBs, To)
 	     error_status = Err, 
 	     error_index  = EI,
 	     varbinds     = VBs} = PDU when is_list(Idx) ->
-	    io:format("EXPECT received pdu (14): "
-		      "~n   [exp] Type:         ~p"
-		      "~n   ReqId:              ~p"
-		      "~n   [exp] Error Status: ~p"
-		      "~n   [exp] Error Index:  ~p"
-		      "~n   VBs:                ~p"
-		      "~nwhen"
-		      "~n   ExpVBs:             ~p"
-		      "~n", [Type, ReqId, Err, EI, VBs, ExpVBs]),
+	    io_format_expect("received pdu (14): "
+                             "~n   [exp] Type:         ~p"
+                             "~n   ReqId:              ~p"
+                             "~n   [exp] Error Status: ~p"
+                             "~n   [exp] Error Index:  ~p"
+                             "~n   VBs:                ~p"
+                             "~nwhen"
+                             "~n   ExpVBs:             ~p",
+                             [Type, ReqId, Err, EI, VBs, ExpVBs]),
 	    PureVBs = purify_oids(ExpVBs), 
 	    case lists:member(EI, Idx) of
 		true ->
@@ -1214,13 +1244,13 @@ do_expect2(Check, Type, Err, Idx, ExpVBs, To)
 	     error_status = Err2, 
 	     error_index  = Idx2,
 	     varbinds     = VBs2} ->
-	    io:format("EXPECT received unexpected pdu with (15) "
-		      "~n   Type:         ~p"
-		      "~n   ReqId:        ~p"
-		      "~n   Errot status: ~p"
-		      "~n   Error index:  ~p"
-		      "~n   Varbinds:     ~p"
-		      "~n", [Type2, ReqId, Err2, Idx2, VBs2]),
+	    io_format_expect("received unexpected pdu with (15) "
+                             "~n   Type:         ~p"
+                             "~n   ReqId:        ~p"
+                             "~n   Errot status: ~p"
+                             "~n   Error index:  ~p"
+                             "~n   Varbinds:     ~p",
+                             [Type2, ReqId, Err2, Idx2, VBs2]),
 	    {error, 
 	     {unexpected_pdu, 
 	      {Type,  Err,  Idx, purify_oids(ExpVBs)}, 
@@ -1228,9 +1258,8 @@ do_expect2(Check, Type, Err, Idx, ExpVBs, To)
 	      ReqId}};
 	
 	Error ->
-	    io:format("EXPECT received error (16):  "
-		      "~n   Error: ~p"
-		      "~n", [Error]),
+	    io_format_expect("received error (16):  "
+                             "~n   Error: ~p", [Error]),
 	    Error
     end.
 
