@@ -226,22 +226,20 @@ agent_capabilities(Config) when is_list(Config) ->
     put(tname,agent_capabilities),
     p("starting with Config: ~p~n", [Config]),
 
-    SnmpPrivDir    = code:priv_dir(snmp),
+    SnmpPrivDir    = which_priv_dir(snmp),
     SnmpMibsDir    = join(SnmpPrivDir, "mibs"), 
-    OtpMibsPrivDir = code:priv_dir(otp_mibs),
-    OtpMibsMibsDir = join(OtpMibsPrivDir, "mibs"), 
     Dir   = ?config(mib_dir, Config),
     AcMib = join(Dir,"AC-TEST-MIB.mib"),
     ?line {ok, MibFile1} = snmpc:compile(AcMib, [options,
 						 version,
-						 {i,         [SnmpMibsDir, OtpMibsMibsDir]}, 
+						 {i,         [SnmpMibsDir]}, 
 						 {outdir,    Dir}, 
 						 {verbosity, trace}]),
     ?line {ok, Mib1} = snmp_misc:read_mib(MibFile1), 
     ?line {ok, MibFile2} = snmpc:compile(AcMib, [options,
 						 version,
 						 agent_capabilities,
-						 {i,         [SnmpMibsDir, OtpMibsMibsDir]}, 
+						 {i,         [SnmpMibsDir]}, 
 						 {outdir,    Dir}, 
 						 {verbosity, trace}]),
     ?line {ok, Mib2} = snmp_misc:read_mib(MibFile2), 
@@ -269,28 +267,26 @@ module_compliance(Config) when is_list(Config) ->
     put(tname,module_compliance),
     p("starting with Config: ~p~n", [Config]),
 
-    SnmpPrivDir    = code:priv_dir(snmp),
-    SnmpMibsDir    = join(SnmpPrivDir, "mibs"), 
-    OtpMibsPrivDir = code:priv_dir(otp_mibs),
-    OtpMibsMibsDir = join(OtpMibsPrivDir, "mibs"), 
-    Dir   = ?config(mib_dir, Config),
-    AcMib = join(Dir,"MC-TEST-MIB.mib"),
+    SnmpPrivDir = which_priv_dir(snmp),
+    SnmpMibsDir = join(SnmpPrivDir, "mibs"), 
+    Dir         = ?config(mib_dir, Config),
+    AcMib       = join(Dir,"MC-TEST-MIB.mib"),
     ?line {ok, MibFile1} = snmpc:compile(AcMib, [options,
 						 version,
-						 {i,           [SnmpMibsDir, OtpMibsMibsDir]}, 
+						 {i,           [SnmpMibsDir]}, 
 						 {outdir,      Dir}, 
 						 {verbosity,   trace}]),
     ?line {ok, Mib1} = snmp_misc:read_mib(MibFile1), 
     ?line {ok, MibFile2} = snmpc:compile(AcMib, [options,
 						 version,
 						 module_compliance,
-						 {i,           [SnmpMibsDir, OtpMibsMibsDir]}, 
+						 {i,           [SnmpMibsDir]}, 
 						 {outdir,      Dir}, 
 						 {verbosity,   trace}]),
     ?line {ok, Mib2} = snmp_misc:read_mib(MibFile2), 
     MEDiff = Mib2#mib.mes -- Mib1#mib.mes,
     %% This is a rather pathetic test, but it is somthing...
-    io:format("agent_capabilities -> "
+    io:format("module_compliance -> "
 	      "~n   MEDiff: ~p"
 	      "~n   Mib1:   ~p"
 	      "~n   Mib2:   ~p"
@@ -731,6 +727,15 @@ check_desc(Desc1, Desc2) ->
     exit({'description not equal', Desc1, Desc2}).
 
 
+which_priv_dir(App) ->
+    case code:priv_dir(App) of
+        Dir when is_list(Dir) ->
+            Dir;
+        {error, Reason} ->
+            exit({App, priv_dir_not_found, Reason})
+    end.
+    
+
 %% join(Comp) ->
 %%     filename:join(Comp).
 
