@@ -1,7 +1,7 @@
 %
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2004-2017. All Rights Reserved.
+%% Copyright Ericsson AB 2004-2018. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -41,35 +41,51 @@
 	 shell/1, shell/2, shell/3
 	]).
 
-%%% Type exports
--export_type([ssh_daemon_ref/0,
-              ssh_connection_ref/0,
-	      ssh_channel_id/0,
-              role/0,
-              subsystem_spec/0,
-              subsystem_name/0,
-              channel_callback/0,
-              channel_init_args/0,
-              algs_list/0,
-              alg_entry/0,
-              simple_algs/0,
-              double_algs/0
-	     ]).
-
--opaque ssh_daemon_ref()     :: daemon_ref() .
--opaque ssh_connection_ref() :: connection_ref() .
+%%% "Deprecated" types export:
+-export_type([ssh_daemon_ref/0, ssh_connection_ref/0, ssh_channel_id/0]).
+-opaque ssh_daemon_ref()     :: daemon_ref().
+-opaque ssh_connection_ref() :: connection_ref().
 -opaque ssh_channel_id()     :: channel_id().
 
+
+%%% Type exports
+-export_type([daemon_ref/0,
+              connection_ref/0,
+	      channel_id/0,
+              client_options/0, client_option/0,
+              daemon_options/0, daemon_option/0,
+              common_options/0,
+              role/0,
+              subsystem_spec/0,
+              algs_list/0,
+              double_algs/1,
+              modify_algs_list/0,
+              alg_entry/0,
+              kex_alg/0,
+              pubkey_alg/0,
+              cipher_alg/0,
+              mac_alg/0,
+              compression_alg/0,
+              ip_port/0
+	     ]).
+
+
+-opaque daemon_ref()         :: pid() .
+-opaque channel_id()     :: non_neg_integer().
+-type connection_ref()       :: pid().  % should be -opaque, but that gives problems
+
 %%--------------------------------------------------------------------
--spec start() -> ok | {error, term()}.
--spec start(permanent | transient | temporary) -> ok | {error, term()}.
-%%
 %% Description: Starts the ssh application. Default type
 %% is temporary. see application(3)
 %%--------------------------------------------------------------------
+-spec start() -> ok | {error, term()}.
+
 start() ->
     start(temporary).
 
+-spec start(Type) -> ok | {error, term()} when
+      Type :: permanent | transient | temporary .
+
 start(Type) ->
     case application:ensure_all_started(ssh, Type) of
         {ok, _} ->
@@ -79,48 +95,57 @@ start(Type) ->
     end.
 
 %%--------------------------------------------------------------------
--spec stop() -> ok | {error, term()}.
-%%
 %% Description: Stops the ssh application.
 %%--------------------------------------------------------------------
+-spec stop() -> ok | {error, term()}.
+
 stop() ->
     application:stop(ssh).
 
 %%--------------------------------------------------------------------
--spec connect(inet:socket(), proplists:proplist()) -> ok_error(connection_ref()).
-
--spec connect(inet:socket(), proplists:proplist(), timeout()) -> ok_error(connection_ref())
-           ; (string(), inet:port_number(), proplists:proplist()) -> ok_error(connection_ref()).
-
--spec connect(string(), inet:port_number(), proplists:proplist(), timeout()) -> ok_error(connection_ref()).
-
-%%
 %% Description: Starts an ssh connection.
 %%--------------------------------------------------------------------
-connect(Socket, UserOptions) when is_port(Socket),
-                                  is_list(UserOptions) ->
-    connect(Socket, UserOptions, infinity).
+-spec connect(OpenTcpSocket, Options) -> {ok,connection_ref()} | {error,term()} when
+      OpenTcpSocket :: open_socket(),
+      Options :: client_options().
 
-connect(Socket, UserOptions, Timeout) when is_port(Socket),
-                                           is_list(UserOptions) ->
+connect(OpenTcpSocket, Options) when is_port(OpenTcpSocket),
+                                     is_list(Options) ->
+    connect(OpenTcpSocket, Options, infinity).
+
+
+-spec connect(open_socket(), client_options(), timeout()) ->
+                     {ok,connection_ref()} | {error,term()}
+           ; (host(), inet:port_number(), client_options()) ->
+                     {ok,connection_ref()} | {error,term()}.
+
+connect(Socket, UserOptions, NegotiationTimeout) when is_port(Socket),
+                                                      is_list(UserOptions) ->
     case ssh_options:handle_options(client, UserOptions) of
 	{error, Error} ->
 	    {error, Error};
 	Options ->
             case valid_socket_to_use(Socket, ?GET_OPT(transport,Options)) of
 		ok ->
-		    {ok, {Host,_Port}} = inet:sockname(Socket),
+		    {ok, {Host,_Port}} = inet:peername(Socket),
 		    Opts = ?PUT_INTERNAL_OPT([{user_pid,self()}, {host,Host}], Options),
-		    ssh_connection_handler:start_connection(client, Socket, Opts, Timeout);
+		    ssh_connection_handler:start_connection(client, Socket, Opts, NegotiationTimeout);
 		{error,SockError} ->
 		    {error,SockError}
 	    end
     end;
 
-connect(Host, Port, UserOptions) when is_integer(Port),
-                                      Port>0,
-                                      is_list(UserOptions) ->
-    connect(Host, Port, UserOptions, infinity).
+connect(Host, Port, Options) when is_integer(Port),
+                                  Port>0,
+                                  is_list(Options) ->
+    connect(Host, Port, Options, infinity).
+
+
+-spec connect(Host, Port, Options, NegotiationTimeout) -> {ok,connection_ref()} | {error,term()} when
+      Host :: host(),
+      Port :: inet:port_number(),
+      Options :: client_options(),
+      NegotiationTimeout :: timeout().
 
 connect(Host0, Port, UserOptions, Timeout) when is_integer(Port),
                                                Port>0,
@@ -148,7 +173,8 @@ connect(Host0, Port, UserOptions, Timeout) when is_integer(Port),
     end.
 
 %%--------------------------------------------------------------------
--spec close(pid()) -> ok.
+-spec close(ConnectionRef) -> ok | {error,term()} when
+      ConnectionRef :: connection_ref() .
 %%
 %% Description: Closes an ssh connection.
 %%--------------------------------------------------------------------
@@ -156,15 +182,25 @@ close(ConnectionRef) ->
     ssh_connection_handler:stop(ConnectionRef).
 
 %%--------------------------------------------------------------------
--spec connection_info(pid(), [atom()]) -> [{atom(), term()}].
-%%
 %% Description: Retrieves information about a connection.
 %%--------------------------------------------------------------------
-connection_info(ConnectionRef, Options) ->
-    ssh_connection_handler:connection_info(ConnectionRef, Options).
+-spec connection_info(ConnectionRef, Keys) ->  ConnectionInfo when
+      ConnectionRef :: connection_ref(),
+      Keys :: [client_version | server_version | user | peer | sockname],
+      ConnectionInfo :: [{client_version, Version}
+                         | {server_version, Version}
+                         | {user,string()}
+                         | {peer, {inet:hostname(), ip_port()}}
+                         | {sockname, ip_port()}
+                        ],
+      Version :: {ProtocolVersion, VersionString::string()},
+      ProtocolVersion :: {Major::pos_integer(), Minor::non_neg_integer()} .
+
+connection_info(Connection, Options) ->
+    ssh_connection_handler:connection_info(Connection, Options).
 
 %%--------------------------------------------------------------------
--spec channel_info(pid(), channel_id(), [atom()]) -> [{atom(), term()}].
+-spec channel_info(connection_ref(), channel_id(), [atom()]) -> proplists:proplist().
 %%
 %% Description: Retrieves information about a connection.
 %%--------------------------------------------------------------------
@@ -172,19 +208,17 @@ channel_info(ConnectionRef, ChannelId, Options) ->
     ssh_connection_handler:channel_info(ConnectionRef, ChannelId, Options).
 
 %%--------------------------------------------------------------------
--spec daemon(inet:port_number()) ->  ok_error(daemon_ref()).
--spec daemon(inet:port_number()|inet:socket(), proplists:proplist()) -> ok_error(daemon_ref()).
--spec daemon(any | inet:ip_address(), inet:port_number(), proplists:proplist()) -> ok_error(daemon_ref())
-           ;(socket, inet:socket(), proplists:proplist()) -> ok_error(daemon_ref())
-            .
-
 %% Description: Starts a server listening for SSH connections
 %% on the given port.
 %%--------------------------------------------------------------------
+-spec daemon(inet:port_number()) ->  {ok,daemon_ref()} | {error,term()}.
+
 daemon(Port) ->
     daemon(Port, []).
 
 
+-spec daemon(inet:port_number()|open_socket(), daemon_options()) -> {ok,daemon_ref()} | {error,term()}.
+
 daemon(Socket, UserOptions) when is_port(Socket) ->
     try
         #{} = Options = ssh_options:handle_options(server, UserOptions),
@@ -227,30 +261,47 @@ daemon(Port, UserOptions) when 0 =< Port, Port =< 65535 ->
     daemon(any, Port, UserOptions).
 
 
+-spec daemon(any | inet:ip_address(), inet:port_number(), daemon_options()) -> {ok,daemon_ref()} | {error,term()}
+           ;(socket, open_socket(), daemon_options()) -> {ok,daemon_ref()} | {error,term()}
+            .
+
 daemon(Host0, Port0, UserOptions0) when 0 =< Port0, Port0 =< 65535,
                                         Host0 == any ; Host0 == loopback ; is_tuple(Host0) ->
     try
         {Host1, UserOptions} = handle_daemon_args(Host0, UserOptions0),
         #{} = Options0 = ssh_options:handle_options(server, UserOptions),
-
-        {{Host,Port}, ListenSocket} =
-            open_listen_socket(Host1, Port0, Options0),
-
-        %% Now Host,Port is what to use for the supervisor to register its name,
-        %% and ListenSocket is for listening on connections. But it is still owned
-        %% by self()...
-
-        finalize_start(Host, Port, ?GET_OPT(profile, Options0),
-                       ?PUT_INTERNAL_OPT({lsocket,{ListenSocket,self()}}, Options0),
-                       fun(Opts, Result) ->
-                               {_, Callback, _} = ?GET_OPT(transport, Opts),
-                               receive
-                                   {request_control, ListenSocket, ReqPid} ->
-                                       ok = Callback:controlling_process(ListenSocket, ReqPid),
-                                       ReqPid ! {its_yours,ListenSocket},
-                                       Result
-                               end
-                       end)
+        {open_listen_socket(Host1, Port0, Options0), Options0}
+    of
+        {{{Host,Port}, ListenSocket}, Options1} ->
+            try
+                %% Now Host,Port is what to use for the supervisor to register its name,
+                %% and ListenSocket is for listening on connections. But it is still owned
+                %% by self()...
+                finalize_start(Host, Port, ?GET_OPT(profile, Options1),
+                               ?PUT_INTERNAL_OPT({lsocket,{ListenSocket,self()}}, Options1),
+                               fun(Opts, Result) ->
+                                       {_, Callback, _} = ?GET_OPT(transport, Opts),
+                                       receive
+                                           {request_control, ListenSocket, ReqPid} ->
+                                               ok = Callback:controlling_process(ListenSocket, ReqPid),
+                                               ReqPid ! {its_yours,ListenSocket},
+                                               Result
+                                       end
+                               end)
+            of
+                {error,Err} ->
+                    close_listen_socket(ListenSocket, Options1),
+                    {error,Err};
+                OK ->
+                    OK
+            catch
+                error:Error ->
+                    close_listen_socket(ListenSocket, Options1),
+                    error(Error);
+                exit:Exit ->
+                    close_listen_socket(ListenSocket, Options1),
+                    exit(Exit)
+            end
     catch
         throw:bad_fd ->
             {error,bad_fd};
@@ -267,10 +318,13 @@ daemon(Host0, Port0, UserOptions0) when 0 =< Port0, Port0 =< 65535,
 daemon(_, _, _) ->
     {error, badarg}.
 
-
-
 %%--------------------------------------------------------------------
--spec daemon_info(daemon_ref()) -> ok_error( [{atom(), term()}] ).
+-spec daemon_info(Daemon) -> {ok, DaemonInfo} | {error,term()} when
+      Daemon :: daemon_ref(),
+      DaemonInfo :: [  {ip, inet:ip_address()}
+                       | {port, inet:port_number()}
+                       | {profile, term()}
+                    ].
 
 daemon_info(Pid) ->
     case catch ssh_system_sup:acceptor_supervisor(Pid) of
@@ -293,16 +347,23 @@ daemon_info(Pid) ->
     end.
 
 %%--------------------------------------------------------------------
--spec stop_listener(daemon_ref()) -> ok.
--spec stop_listener(inet:ip_address(), inet:port_number()) -> ok.
-%%
 %% Description: Stops the listener, but leaves
 %% existing connections started by the listener up and running.
 %%--------------------------------------------------------------------
+-spec stop_listener(daemon_ref()) -> ok.
+
 stop_listener(SysSup) ->
     ssh_system_sup:stop_listener(SysSup).
+
+
+-spec stop_listener(inet:ip_address(), inet:port_number()) -> ok.
+
 stop_listener(Address, Port) ->
     stop_listener(Address, Port, ?DEFAULT_PROFILE).
+
+
+-spec stop_listener(any|inet:ip_address(), inet:port_number(), term()) -> ok.
+
 stop_listener(any, Port, Profile) ->
     map_ip(fun(IP) ->
                    ssh_system_sup:stop_listener(IP, Port, Profile) 
@@ -313,17 +374,23 @@ stop_listener(Address, Port, Profile) ->
            end, {address,Address}).
 
 %%--------------------------------------------------------------------
--spec stop_daemon(daemon_ref()) -> ok.
--spec stop_daemon(inet:ip_address(), inet:port_number()) -> ok.
--spec stop_daemon(inet:ip_address(), inet:port_number(), atom()) -> ok.
-%%
 %% Description: Stops the listener and all connections started by
 %% the listener.
 %%--------------------------------------------------------------------
+-spec stop_daemon(DaemonRef::daemon_ref()) -> ok.
+
 stop_daemon(SysSup) ->
     ssh_system_sup:stop_system(SysSup).
+
+
+-spec stop_daemon(inet:ip_address(), inet:port_number()) -> ok.
+
 stop_daemon(Address, Port) ->
     stop_daemon(Address, Port, ?DEFAULT_PROFILE).
+
+
+-spec stop_daemon(any|inet:ip_address(), inet:port_number(), atom()) -> ok.
+
 stop_daemon(any, Port, Profile) ->
     map_ip(fun(IP) ->
                    ssh_system_sup:stop_system(IP, Port, Profile) 
@@ -334,33 +401,37 @@ stop_daemon(Address, Port, Profile) ->
            end, {address,Address}).
 
 %%--------------------------------------------------------------------
--spec shell(inet:socket() | string()) ->  _.
--spec shell(inet:socket() | string(), proplists:proplist()) ->  _.
--spec shell(string(), inet:port_number(), proplists:proplist()) ->  _.
-
-%%   Host = string()
-%%   Port = integer()
-%%   Options = [{Option, Value}]
-%%
 %% Description: Starts an interactive shell to an SSH server on the
 %% given <Host>. The function waits for user input,
 %% and will not return until the remote shell is ended.(e.g. on
 %% exit from the shell)
 %%--------------------------------------------------------------------
+-spec shell(open_socket() | host()) ->  _.
+
 shell(Socket) when is_port(Socket) ->
     shell(Socket, []);
 shell(Host) ->
     shell(Host, ?SSH_DEFAULT_PORT, []).
 
+
+-spec shell(open_socket() | host(), client_options()) ->  _.
+
 shell(Socket, Options) when is_port(Socket) ->
     start_shell( connect(Socket, Options) );
 shell(Host, Options) ->
     shell(Host, ?SSH_DEFAULT_PORT, Options).
 
+
+-spec shell(Host, Port, Options) -> _ when
+      Host :: host(),
+      Port :: inet:port_number(),
+      Options :: client_options() .
+
 shell(Host, Port, Options) ->
     start_shell( connect(Host, Port, Options) ).
 
 
+
 start_shell({ok, ConnectionRef}) ->
     case ssh_connection:session_channel(ConnectionRef, infinity) of
 	{ok,ChannelId}  ->
@@ -368,11 +439,17 @@ start_shell({ok, ConnectionRef}) ->
 	    Args = [{channel_cb, ssh_shell},
 		    {init_args,[ConnectionRef, ChannelId]},
 		    {cm, ConnectionRef}, {channel_id, ChannelId}],
-	    {ok, State} = ssh_channel:init([Args]),
-	    ssh_channel:enter_loop(State);
+	    {ok, State} = ssh_client_channel:init([Args]),
+            try
+                ssh_client_channel:enter_loop(State)
+            catch
+                exit:normal ->
+                    ok
+            end;
 	Error ->
 	    Error
     end;
+
 start_shell(Error) ->
     Error.
 
@@ -383,7 +460,7 @@ default_algorithms() ->
     ssh_transport:default_algorithms().
 
 %%--------------------------------------------------------------------
--spec chk_algos_opts(list(any())) -> algs_list() .
+-spec chk_algos_opts(client_options()|daemon_options()) -> internal_options() | {error,term()}.
 %%--------------------------------------------------------------------
 chk_algos_opts(Opts) ->
     case lists:foldl(
@@ -459,6 +536,15 @@ open_listen_socket(_Host0, Port0, Options0) ->
     {ok,{LHost,LPort}} = inet:sockname(LSock),
     {{LHost,LPort}, LSock}.
 
+%%%----------------------------------------------------------------
+close_listen_socket(ListenSocket, Options) ->
+    try
+        {_, Callback, _} = ?GET_OPT(transport, Options),
+        Callback:close(ListenSocket)
+    catch
+        _C:_E -> ok
+    end.
+
 %%%----------------------------------------------------------------
 finalize_start(Host, Port, Profile, Options0, F) ->
     try
