@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2018-2018. All Rights Reserved.
+%% Copyright Ericsson AB 2018-2019. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -38,19 +38,24 @@
 
 -define(SERVER, ?MODULE).
 
--record(data, {connection_pid,
-               connection_states = #{},
-               role,
-               socket,
-               socket_options,
-               tracker,
-               protocol_cb,
-               transport_cb,
-               negotiated_version,
-               renegotiate_at,
-               connection_monitor,
-               dist_handle
-              }).
+-record(static,
+        {connection_pid,
+         role,
+         socket,
+         socket_options,
+         tracker,
+         transport_cb,
+         negotiated_version,
+         renegotiate_at,
+         connection_monitor,
+         dist_handle,
+         log_level
+        }).
+
+-record(data,
+        {static = #static{},
+         connection_states = #{}
+        }).
 
 %%%===================================================================
 %%% API
@@ -171,6 +176,10 @@ dist_tls_socket(Pid) ->
 callback_mode() -> 
     state_functions.
 
+
+-define(HANDLE_COMMON,
+        ?FUNCTION_NAME(Type, Msg, StateData) ->
+               handle_common(Type, Msg, StateData)).
 %%--------------------------------------------------------------------
 -spec init(Args :: term()) ->
                   gen_statem:init_result(atom()).
@@ -192,39 +201,37 @@ init({call, From}, {Pid, #{current_write := WriteState,
                            socket := Socket,
                            socket_options := SockOpts,
                            tracker := Tracker,
-                           protocol_cb := Connection,
                            transport_cb := Transport,
                            negotiated_version := Version,
-                           renegotiate_at := RenegotiateAt}}, 
-     #data{connection_states = ConnectionStates} = StateData0) ->    
+                           renegotiate_at := RenegotiateAt,
+                           log_level := LogLevel}},
+     #data{connection_states = ConnectionStates, static = Static0} = StateData0) ->
     Monitor = erlang:monitor(process, Pid),
     StateData = 
-        StateData0#data{connection_pid = Pid,
-                        connection_monitor = Monitor,
-                        connection_states = 
-                            ConnectionStates#{current_write => WriteState},
-                        role = Role,
-                        socket = Socket,
-                        socket_options = SockOpts,
-                        tracker = Tracker,
-                        protocol_cb = Connection,
-                        transport_cb = Transport,
-                        negotiated_version = Version,
-                        renegotiate_at = RenegotiateAt},          
+        StateData0#data{connection_states = ConnectionStates#{current_write => WriteState},
+                        static = Static0#static{connection_pid = Pid,
+                                                connection_monitor = Monitor,
+                                                role = Role,
+                                                socket = Socket,
+                                                socket_options = SockOpts,
+                                                tracker = Tracker,
+                                                transport_cb = Transport,
+                                                negotiated_version = Version,
+                                                renegotiate_at = RenegotiateAt,
+                                                log_level = LogLevel}},
     {next_state, handshake, StateData, [{reply, From, ok}]};
-init(info, Msg, StateData) -> 
-    handle_info(Msg, ?FUNCTION_NAME, StateData).
+init(_, _, _) ->
+    %% Just in case anything else sneeks through
+    {keep_state_and_data, [postpone]}.
+
 %%--------------------------------------------------------------------
 -spec connection(gen_statem:event_type(),
            Msg :: term(),
            StateData :: term()) ->
                   gen_statem:event_handler_result(atom()).
 %%--------------------------------------------------------------------
-connection({call, From}, renegotiate, 
-           #data{connection_states = #{current_write := Write}} = StateData) ->
-    {next_state, handshake, StateData, [{reply, From, {ok, Write}}]};
 connection({call, From}, {application_data, AppData}, 
-           #data{socket_options = #socket_options{packet = Packet}} =
+           #data{static = #static{socket_options = #socket_options{packet = Packet}}} =
                StateData) ->
     case encode_packet(Packet, AppData) of
         {error, _} = Error ->
@@ -232,40 +239,40 @@ connection({call, From}, {application_data, AppData},
         Data ->
             send_application_data(Data, From, ?FUNCTION_NAME, StateData)
     end;
-connection({call, From}, {set_opts, _} = Call, StateData) ->
-    handle_call(From, Call, ?FUNCTION_NAME, StateData);
-connection({call, From}, dist_get_tls_socket, 
-           #data{protocol_cb = Connection, 
-                 transport_cb = Transport,
-                 socket = Socket,
-                 connection_pid = Pid, 
-                 tracker = Tracker} = StateData) ->
-    TLSSocket = Connection:socket([Pid, self()], Transport, Socket, Connection, Tracker),
-    {next_state, ?FUNCTION_NAME, StateData, [{reply, From, {ok, TLSSocket}}]};
-connection({call, From}, {dist_handshake_complete, _Node, DHandle},
-           #data{connection_pid = Pid,
-                 socket_options = #socket_options{packet = Packet}} =
-               StateData) ->
-    ok = erlang:dist_ctrl_input_handler(DHandle, Pid),
-    ok = ssl_connection:dist_handshake_complete(Pid, DHandle),
-    %% From now on we execute on normal priority
-    process_flag(priority, normal),
-    {next_state, ?FUNCTION_NAME, StateData#data{dist_handle = DHandle},
-     [{reply, From, ok}
-      | case dist_data(DHandle, Packet) of
-            [] ->
-                [];
-            Data ->
-                [{next_event, internal,
-                  {application_packets,{self(),undefined},Data}}]
-        end]};
 connection({call, From}, {ack_alert, #alert{} = Alert}, StateData0) ->
     StateData = send_tls_alert(Alert, StateData0),
     {next_state, ?FUNCTION_NAME, StateData,
      [{reply,From,ok}]};
-connection({call, From}, downgrade, #data{connection_states = 
+connection({call, From}, renegotiate,
+           #data{connection_states = #{current_write := Write}} = StateData) ->
+    {next_state, handshake, StateData, [{reply, From, {ok, Write}}]};
+connection({call, From}, downgrade, #data{connection_states =
                                               #{current_write := Write}} = StateData) ->
     {next_state, death_row, StateData, [{reply,From, {ok, Write}}]};
+connection({call, From}, {set_opts, Opts}, StateData) ->
+    handle_set_opts(From, Opts, StateData);
+connection({call, From}, dist_get_tls_socket, 
+           #data{static = #static{transport_cb = Transport,
+                                  socket = Socket,
+                                  connection_pid = Pid,
+                                  tracker = Tracker}} = StateData) ->
+    TLSSocket = tls_connection:socket([Pid, self()], Transport, Socket, Tracker),
+    {next_state, ?FUNCTION_NAME, StateData, [{reply, From, {ok, TLSSocket}}]};
+connection({call, From}, {dist_handshake_complete, _Node, DHandle},
+           #data{static = #static{connection_pid = Pid} = Static} = StateData) ->
+    ok = erlang:dist_ctrl_input_handler(DHandle, Pid),
+    ok = ssl_connection:dist_handshake_complete(Pid, DHandle),
+    %% From now on we execute on normal priority
+    process_flag(priority, normal),
+    {keep_state, StateData#data{static = Static#static{dist_handle = DHandle}},
+     [{reply,From,ok}|
+      case dist_data(DHandle) of
+          [] ->
+              [];
+          Data ->
+              [{next_event, internal,
+               {application_packets,{self(),undefined},erlang:iolist_to_iovec(Data)}}]
+      end]};
 connection(internal, {application_packets, From, Data}, StateData) ->
     send_application_data(Data, From, ?FUNCTION_NAME, StateData);
 %%
@@ -273,29 +280,26 @@ connection(cast, #alert{} = Alert, StateData0) ->
     StateData = send_tls_alert(Alert, StateData0),
     {next_state, ?FUNCTION_NAME, StateData};
 connection(cast, {new_write, WritesState, Version}, 
-           #data{connection_states = ConnectionStates0} = StateData) -> 
+           #data{connection_states = ConnectionStates, static = Static} = StateData) ->
     {next_state, connection, 
      StateData#data{connection_states = 
-                        ConnectionStates0#{current_write => WritesState},
-                    negotiated_version = Version}};
+                        ConnectionStates#{current_write => WritesState},
+                    static = Static#static{negotiated_version = Version}}};
 %%
-connection(info, dist_data,
-           #data{dist_handle = DHandle,
-                 socket_options = #socket_options{packet = Packet}} =
-               StateData) ->
-    {next_state, ?FUNCTION_NAME, StateData,
-      case dist_data(DHandle, Packet) of
+connection(info, dist_data, #data{static = #static{dist_handle = DHandle}}) ->
+    {keep_state_and_data,
+      case dist_data(DHandle) of
           [] ->
               [];
           Data ->
               [{next_event, internal,
-               {application_packets,{self(),undefined},Data}}]
+               {application_packets,{self(),undefined},erlang:iolist_to_iovec(Data)}}]
       end};
 connection(info, tick, StateData) ->  
     consume_ticks(),
-    {next_state, ?FUNCTION_NAME, StateData, 
-     [{next_event, {call, {self(), undefined}},
-       {application_data, <<>>}}]};
+    Data = [<<0:32>>], % encode_packet(4, <<>>)
+    From = {self(), undefined},
+    send_application_data(Data, From, ?FUNCTION_NAME, StateData);
 connection(info, {send, From, Ref, Data}, _StateData) -> 
     %% This is for testing only!
     %%
@@ -304,29 +308,37 @@ connection(info, {send, From, Ref, Data}, _StateData) ->
     From ! {Ref, ok},
     {keep_state_and_data,
      [{next_event, {call, {self(), undefined}},
-       {application_data, iolist_to_binary(Data)}}]};
-connection(info, Msg, StateData) -> 
-    handle_info(Msg, ?FUNCTION_NAME, StateData).
+       {application_data, erlang:iolist_to_iovec(Data)}}]};
+?HANDLE_COMMON.
+
 %%--------------------------------------------------------------------
 -spec handshake(gen_statem:event_type(),
                   Msg :: term(),
                   StateData :: term()) ->
                          gen_statem:event_handler_result(atom()).
 %%--------------------------------------------------------------------
-handshake({call, From}, {set_opts, _} = Call, StateData) ->
-    handle_call(From, Call, ?FUNCTION_NAME, StateData);
+handshake({call, From}, {set_opts, Opts}, StateData) ->
+    handle_set_opts(From, Opts, StateData);
 handshake({call, _}, _, _) ->
+    %% Postpone all calls to the connection state
     {keep_state_and_data, [postpone]};
-handshake(cast, {new_write, WritesState, Version}, 
-          #data{connection_states = ConnectionStates0} = StateData) ->
-    {next_state, connection, 
-     StateData#data{connection_states = 
-                        ConnectionStates0#{current_write => WritesState},
-                   negotiated_version = Version}};
 handshake(internal, {application_packets,_,_}, _) ->
     {keep_state_and_data, [postpone]};
-handshake(info, Msg, StateData) -> 
-    handle_info(Msg, ?FUNCTION_NAME, StateData).
+handshake(cast, {new_write, WritesState, Version}, 
+          #data{connection_states = ConnectionStates, static = Static} = StateData) ->
+    {next_state, connection, 
+     StateData#data{connection_states = ConnectionStates#{current_write => WritesState},
+                    static = Static#static{negotiated_version = Version}}};
+handshake(info, dist_data, _) ->
+    {keep_state_and_data, [postpone]};
+handshake(info, tick, _) ->
+    %% Ignore - data is sent anyway during handshake
+    consume_ticks(),
+    keep_state_and_data;
+handshake(info, {send, _, _, _}, _) ->
+    %% Testing only, OTP distribution test suites...
+    {keep_state_and_data, [postpone]};
+?HANDLE_COMMON.
 
 %%--------------------------------------------------------------------
 -spec death_row(gen_statem:event_type(),
@@ -361,54 +373,76 @@ code_change(_OldVsn, State, Data, _Extra) ->
 %%%===================================================================
 %%% Internal functions
 %%%===================================================================
-handle_call(From, {set_opts, Opts}, StateName, #data{socket_options = SockOpts} = StateData) ->
-    {next_state, StateName, StateData#data{socket_options = set_opts(SockOpts, Opts)}, [{reply, From, ok}]}.
-        
-handle_info({'DOWN', Monitor, _, _, Reason}, _, 
-            #data{connection_monitor = Monitor,
-                  dist_handle = Handle} = StateData) when Handle =/= undefined->
-    {next_state, death_row, StateData, [{state_timeout, 5000, Reason}]};
-handle_info({'DOWN', Monitor, _, _, _}, _, 
-            #data{connection_monitor = Monitor} = StateData) ->
+
+handle_set_opts(
+  From, Opts, #data{static = #static{socket_options = SockOpts} = Static} = StateData) ->
+    {keep_state, StateData#data{static = Static#static{socket_options = set_opts(SockOpts, Opts)}},
+     [{reply, From, ok}]}.
+
+handle_common(
+  {call, From}, {set_opts, Opts},
+  #data{static = #static{socket_options = SockOpts} = Static} = StateData) ->
+    {keep_state, StateData#data{static = Static#static{socket_options = set_opts(SockOpts, Opts)}},
+     [{reply, From, ok}]};
+handle_common(
+  info, {'DOWN', Monitor, _, _, Reason},
+  #data{static = #static{connection_monitor = Monitor,
+                         dist_handle = Handle}} = StateData) when Handle =/= undefined ->
+    {next_state, death_row, StateData,
+     [{state_timeout, 5000, Reason}]};
+handle_common(
+  info, {'DOWN', Monitor, _, _, _},
+  #data{static = #static{connection_monitor = Monitor}} = StateData) ->
     {stop, normal, StateData};
-handle_info(_,_,_) ->
+handle_common(info, Msg, _) ->
+    Report =
+        io_lib:format("TLS sender: Got unexpected info: ~p ~n", [Msg]),
+    error_logger:info_report(Report),
+    keep_state_and_data;
+handle_common(Type, Msg, _) ->
+    Report =
+        io_lib:format(
+          "TLS sender: Got unexpected event: ~p ~n", [{Type,Msg}]),
+    error_logger:error_report(Report),
     keep_state_and_data.
 
-send_tls_alert(Alert, #data{negotiated_version = Version,
-                            socket = Socket,
-                            protocol_cb = Connection, 
-                            transport_cb = Transport,
-                            connection_states = ConnectionStates0} = StateData0) ->
+send_tls_alert(#alert{} = Alert,
+               #data{static = #static{negotiated_version = Version,
+                                      socket = Socket,
+                                      transport_cb = Transport,
+                                      log_level = LogLevel},
+                     connection_states = ConnectionStates0} = StateData0) ->
     {BinMsg, ConnectionStates} =
-	Connection:encode_alert(Alert, Version, ConnectionStates0),
-    Connection:send(Transport, Socket, BinMsg),
+	tls_record:encode_alert_record(Alert, Version, ConnectionStates0),
+    tls_socket:send(Transport, Socket, BinMsg),
+    ssl_logger:debug(LogLevel, outbound, 'record', BinMsg),
     StateData0#data{connection_states = ConnectionStates}.
 
 send_application_data(Data, From, StateName,
-		       #data{connection_pid = Pid,
-                             socket = Socket,
-                             dist_handle = DistHandle,
-                             negotiated_version = Version,
-                             protocol_cb = Connection,
-                             transport_cb = Transport,
-                             connection_states = ConnectionStates0,
-                             renegotiate_at = RenegotiateAt} = StateData0) ->
+		       #data{static = #static{connection_pid = Pid,
+                                              socket = Socket,
+                                              dist_handle = DistHandle,
+                                              negotiated_version = Version,
+                                              transport_cb = Transport,
+                                              renegotiate_at = RenegotiateAt,
+                                              log_level = LogLevel},
+                             connection_states = ConnectionStates0} = StateData0) ->
     case time_to_renegotiate(Data, ConnectionStates0, RenegotiateAt) of
 	true ->
 	    ssl_connection:internal_renegotiation(Pid, ConnectionStates0),
             {next_state, handshake, StateData0, 
              [{next_event, internal, {application_packets, From, Data}}]};
 	false ->
-	    {Msgs, ConnectionStates} =
-                Connection:encode_data(
-                  iolist_to_binary(Data), Version, ConnectionStates0),
+	    {Msgs, ConnectionStates} = tls_record:encode_data(Data, Version, ConnectionStates0),
             StateData = StateData0#data{connection_states = ConnectionStates},
-	    case Connection:send(Transport, Socket, Msgs) of
+	    case tls_socket:send(Transport, Socket, Msgs) of
                 ok when DistHandle =/=  undefined ->
+                    ssl_logger:debug(LogLevel, outbound, 'record', Msgs),
                     {next_state, StateName, StateData, []};
                 Reason when DistHandle =/= undefined ->
                     {next_state, death_row, StateData, [{state_timeout, 5000, Reason}]};
                 ok ->
+                    ssl_logger:debug(LogLevel, outbound, 'record', Msgs),
                     {next_state, StateName, StateData,  [{reply, From, ok}]};
                 Result ->
                     {next_state, StateName, StateData,  [{reply, From, Result}]}
@@ -419,9 +453,9 @@ send_application_data(Data, From, StateName,
 encode_packet(Packet, Data) ->
     Len = iolist_size(Data),
     case Packet of
-        1 when Len < (1 bsl 8) ->  [<<Len:8>>,Data];
-        2 when Len < (1 bsl 16) -> [<<Len:16>>,Data];
-        4 when Len < (1 bsl 32) -> [<<Len:32>>,Data];
+        1 when Len < (1 bsl 8) ->  [<<Len:8>>|Data];
+        2 when Len < (1 bsl 16) -> [<<Len:16>>|Data];
+        4 when Len < (1 bsl 32) -> [<<Len:32>>|Data];
         N when N =:= 1; N =:= 2; N =:= 4 ->
             {error,
              {badarg, {packet_to_large, Len, (1 bsl (Packet bsl 3)) - 1}}};
@@ -458,22 +492,30 @@ call(FsmPid, Event) ->
 	    {error, closed}
     end.
 
-%%---------------Erlang distribution --------------------------------------
+%%-------------- Erlang distribution helpers ------------------------------
 
-dist_data(DHandle, Packet) ->
+dist_data(DHandle) ->
     case erlang:dist_ctrl_get_data(DHandle) of
         none ->
             erlang:dist_ctrl_get_data_notification(DHandle),
             [];
-        Data ->
-            %% This is encode_packet(4, Data) without Len check
-            %% since the emulator will always deliver a Data
-            %% smaller than 4 GB, and the distribution will
-            %% therefore always have to use {packet,4}
+        %% This is encode_packet(4, Data) without Len check
+        %% since the emulator will always deliver a Data
+        %% smaller than 4 GB, and the distribution will
+        %% therefore always have to use {packet,4}
+        Data when is_binary(Data) ->
+            Len = byte_size(Data),
+            [[<<Len:32>>,Data]|dist_data(DHandle)];
+        [BA,BB] = Data ->
+            Len = byte_size(BA) + byte_size(BB),
+            [[<<Len:32>>|Data]|dist_data(DHandle)];
+        Data when is_list(Data) ->
             Len = iolist_size(Data),
-            [<<Len:32>>,Data|dist_data(DHandle, Packet)]
+            [[<<Len:32>>|Data]|dist_data(DHandle)]
     end.
 
+
+%% Empty the inbox from distribution ticks - do not let them accumulate
 consume_ticks() ->
     receive tick -> 
             consume_ticks()
