@@ -89,7 +89,8 @@ tests() ->
      critical_extension_verify_server,
      critical_extension_verify_none,
      customize_hostname_check,
-     incomplete_chain
+     incomplete_chain,
+     long_chain
     ].
 
 error_handling_tests()->
@@ -147,6 +148,7 @@ init_per_testcase(_TestCase, Config) ->
     ssl:stop(),
     ssl:start(),
     ssl_test_lib:ct_log_supported_protocol_versions(Config),
+    ct:pal(" ~p", [ dtls_record:supported_protocol_versions()]),
     ct:timetrap({seconds, 10}),
     Config.
 
@@ -298,15 +300,8 @@ server_require_peer_cert_fail(Config) when is_list(Config) ->
 					      {host, Hostname},
 					      {from, self()},
 					      {options, [{active, Active} | BadClientOpts]}]),
-    receive
-	{Server, {error, {tls_alert, "handshake failure"}}} ->
-	    receive
-		{Client, {error, {tls_alert, "handshake failure"}}} ->
-		    ok;
-		{Client, {error, closed}} ->
-		    ok
-	    end
-    end.
+
+    ssl_test_lib:check_server_alert(Server, Client, handshake_failure).
 
 %%--------------------------------------------------------------------
 server_require_peer_cert_empty_ok() ->
@@ -365,15 +360,8 @@ server_require_peer_cert_partial_chain(Config) when is_list(Config) ->
 					      {options, [{active, Active},
 							 {cacerts, [RootCA]} |
 							 proplists:delete(cacertfile, ClientOpts)]}]),
-    receive
-	{Server, {error, {tls_alert, "unknown ca"}}} ->
-	    receive
-		{Client, {error, {tls_alert, "unknown ca"}}} ->
-		    ok;
-		{Client, {error, closed}} ->
-		    ok
-	    end
-    end.
+    ssl_test_lib:check_server_alert(Server, Client, unknown_ca).
+
 %%--------------------------------------------------------------------
 server_require_peer_cert_allow_partial_chain() ->
     [{doc, "Server trusts intermediat CA and accepts a partial chain. (partial_chain option)"}].
@@ -446,17 +434,7 @@ server_require_peer_cert_do_not_allow_partial_chain(Config) when is_list(Config)
 					      {from, self()},
 					      {mfa, {ssl_test_lib, no_result, []}},
 					      {options, ClientOpts}]),
-
-    receive
-	 {Server, {error, {tls_alert, "unknown ca"}}} ->
-	    receive
-		{Client, {error, {tls_alert, "unknown ca"}}} ->
-		    ok;
-		{Client, {error, closed}} ->
-		    ok
-	    end
-    end.
-
+    ssl_test_lib:check_server_alert(Server, Client, unknown_ca).
  %%--------------------------------------------------------------------
 server_require_peer_cert_partial_chain_fun_fail() ->
     [{doc, "If parial_chain fun crashes, treat it as if it returned unkown_ca"}].
@@ -471,7 +449,7 @@ server_require_peer_cert_partial_chain_fun_fail(Config) when is_list(Config) ->
     [{_,_,_}, {_, IntermidiateCA, _} | _] = public_key:pem_decode(ServerCAs),
 
     PartialChain =  fun(_CertChain) ->
-                            ture = false %% crash on purpose
+                            true = false %% crash on purpose
 		    end,
 
     Server = ssl_test_lib:start_server_error([{node, ServerNode}, {port, 0},
@@ -487,16 +465,7 @@ server_require_peer_cert_partial_chain_fun_fail(Config) when is_list(Config) ->
 					      {from, self()},
 					      {mfa, {ssl_test_lib, no_result, []}},
 					      {options, ClientOpts}]),
-
-    receive
-	 {Server, {error, {tls_alert, "unknown ca"}}} ->
-	    receive
-		{Client, {error, {tls_alert, "unknown ca"}}} ->
-		    ok;
-		{Client, {error, closed}} ->
-		    ok
-	    end
-    end.
+    ssl_test_lib:check_server_alert(Server, Client, unknown_ca).
 
 %%--------------------------------------------------------------------
 verify_fun_always_run_client() ->
@@ -514,7 +483,7 @@ verify_fun_always_run_client(Config) when is_list(Config) ->
     Port  = ssl_test_lib:inet_port(Server),
 
     %% If user verify fun is called correctly we fail the connection.
-    %% otherwise we can not tell this case apart form where we miss
+    %% otherwise we cannot tell this case apart form where we miss
     %% to call users verify fun
     FunAndState =  {fun(_,{extension, _}, UserState) ->
 			    {unknown, UserState};
@@ -535,14 +504,8 @@ verify_fun_always_run_client(Config) when is_list(Config) ->
 					       [{verify, verify_peer},
 						{verify_fun, FunAndState}
 						| ClientOpts]}]),
-    %% Server error may be {tls_alert,"handshake failure"} or closed depending on timing
-    %% this is not a bug it is a circumstance of how tcp works!
-    receive
-	{Server, ServerError} ->
-	    ct:log("Server Error ~p~n", [ServerError])
-    end,
 
-    ssl_test_lib:check_result(Client, {error, {tls_alert, "handshake failure"}}).
+    ssl_test_lib:check_client_alert(Server, Client, handshake_failure).
 
 %%--------------------------------------------------------------------
 verify_fun_always_run_server() ->
@@ -553,7 +516,7 @@ verify_fun_always_run_server(Config) when is_list(Config) ->
     {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
 
     %% If user verify fun is called correctly we fail the connection.
-    %% otherwise we can not tell this case apart form where we miss
+    %% otherwise we cannot tell this case apart form where we miss
     %% to call users verify fun
     FunAndState =  {fun(_,{extension, _}, UserState) ->
 			    {unknown, UserState};
@@ -581,16 +544,8 @@ verify_fun_always_run_server(Config) when is_list(Config) ->
 					      {mfa, {ssl_test_lib,
 						     no_result, []}},
 					      {options, ClientOpts}]),
-
-    %% Client error may be {tls_alert, "handshake failure" } or closed depending on timing
-    %% this is not a bug it is a circumstance of how tcp works!
-    receive
-	{Client, ClientError} ->
-	    ct:log("Client Error ~p~n", [ClientError])
-    end,
-
-    ssl_test_lib:check_result(Server, {error, {tls_alert, "handshake failure"}}).
-
+    
+    ssl_test_lib:check_client_alert(Server, Client, handshake_failure).
 %%--------------------------------------------------------------------
 
 cert_expired() ->
@@ -620,8 +575,7 @@ cert_expired(Config) when is_list(Config) ->
 					      {from, self()},
 					      {options, [{verify, verify_peer}, {active, Active}  | ClientOpts]}]),    
     
-    ssl_test_lib:check_result(Server, {error, {tls_alert, "certificate expired"}},
-                              Client, {error, {tls_alert, "certificate expired"}}).
+    ssl_test_lib:check_client_alert(Server, Client, certificate_expired).
 
 two_digits_str(N) when N < 10 ->
     lists:flatten(io_lib:format("0~p", [N]));
@@ -727,12 +681,8 @@ critical_extension_verify_server(Config) when is_list(Config) ->
                 {options, [{verify, verify_none}, {active, Active} | ClientOpts]}]),
 
     %% This certificate has a critical extension that we don't
-    %% understand.  Therefore, verification should fail.      
-
-    ssl_test_lib:check_result(Server, {error, {tls_alert, "unsupported certificate"}},
-                              Client, {error, {tls_alert, "unsupported certificate"}}),
-    
-    ssl_test_lib:close(Server).
+    %% understand.  Therefore, verification should fail.          
+    ssl_test_lib:check_server_alert(Server, Client, unsupported_certificate).
 %%--------------------------------------------------------------------
 
 critical_extension_verify_client() ->
@@ -763,12 +713,7 @@ critical_extension_verify_client(Config) when is_list(Config) ->
                 {mfa, {ssl_test_lib, ReceiveFunction, []}},
                 {options, [{verify, verify_peer}, {active, Active} | ClientOpts]}]),
 
-    %% This certificate has a critical extension that we don't
-    %% understand.  Therefore, verification should fail.
-    ssl_test_lib:check_result(Server, {error, {tls_alert, "unsupported certificate"}},
-                              Client, {error, {tls_alert, "unsupported certificate"}}),
-
-    ssl_test_lib:close(Server).
+    ssl_test_lib:check_client_alert(Server, Client, unsupported_certificate).
 
 %%--------------------------------------------------------------------
 critical_extension_verify_none() ->
@@ -908,10 +853,7 @@ invalid_signature_server(Config) when is_list(Config) ->
 					      {host, Hostname},
 					      {from, self()},
 					      {options, [{verify, verify_peer} | ClientOpts]}]),
-
-    ssl_test_lib:check_result(Server, {error, {tls_alert, "unknown ca"}},
-                              Client, {error, {tls_alert, "unknown ca"}}).
-
+    ssl_test_lib:check_server_alert(Server, Client, unknown_ca).
 %%--------------------------------------------------------------------
 
 invalid_signature_client() ->
@@ -946,9 +888,7 @@ invalid_signature_client(Config) when is_list(Config) ->
 					      {from, self()},
 					      {options, NewClientOpts}]),
 
-    ssl_test_lib:check_result(Server, {error, {tls_alert, "unknown ca"}},
-                              Client, {error, {tls_alert, "unknown ca"}}).
-
+    ssl_test_lib:check_client_alert(Server, Client, unknown_ca).
 
 %%--------------------------------------------------------------------
 
@@ -1034,16 +974,7 @@ unknown_server_ca_fail(Config) when is_list(Config) ->
 					       [{verify, verify_peer},
 						{verify_fun, FunAndState}
 						| ClientOpts]}]),
-    receive
-	{Client, {error, {tls_alert, "unknown ca"}}} ->
-	    receive
-		{Server, {error, {tls_alert, "unknown ca"}}} ->
-		    ok;
-		{Server, {error, closed}} ->
-		    ok
-	    end
-    end.
-
+    ssl_test_lib:check_client_alert(Server, Client, unknown_ca).
 
 %%--------------------------------------------------------------------
 unknown_server_ca_accept_verify_none() ->
@@ -1193,11 +1124,7 @@ customize_hostname_check(Config) when is_list(Config) ->
                                                {mfa, {ssl_test_lib, no_result, []}},
                                                {options, ClientOpts}
                                               ]),    
-    ssl_test_lib:check_result(Client1, {error, {tls_alert, "handshake failure"}},
-                              Server,  {error, {tls_alert, "handshake failure"}}),
-    
-    ssl_test_lib:close(Server),
-    ssl_test_lib:close(Client).
+    ssl_test_lib:check_client_alert(Server, Client1, handshake_failure).
 
 incomplete_chain() ->
     [{doc,"Test option verify_peer"}].
@@ -1231,6 +1158,44 @@ incomplete_chain(Config) when is_list(Config) ->
     ssl_test_lib:close(Server),
     ssl_test_lib:close(Client).
 
+long_chain() ->
+    [{doc,"Test option verify_peer"}].
+long_chain(Config) when is_list(Config) ->      
+    #{server_config := ServerConf,
+      client_config := ClientConf} = public_key:pkix_test_data(#{server_chain => #{root => [{key, ssl_test_lib:hardcode_rsa_key(1)}],
+                                                                                  intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(2)}], 
+                                                                                                    [{key, ssl_test_lib:hardcode_rsa_key(3)}],
+                                                                                                    [{key, ssl_test_lib:hardcode_rsa_key(4)}]],
+                                                                                  peer => [{key, ssl_test_lib:hardcode_rsa_key(5)}]},
+                                                                 client_chain => #{root => [{key, ssl_test_lib:hardcode_rsa_key(3)}], 
+                                                                                  intermediates => [[{key, ssl_test_lib:hardcode_rsa_key(2)}]],
+                                                                                  peer => [{key, ssl_test_lib:hardcode_rsa_key(1)}]}}), 
+    [ServerRoot| _] = ServerCas = proplists:get_value(cacerts, ServerConf),
+    ClientCas = proplists:get_value(cacerts, ClientConf),
+    
+    Active = proplists:get_value(active, Config),
+    ReceiveFunction =  proplists:get_value(receive_function, Config),
+    {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
+    Server = ssl_test_lib:start_server([{node, ServerNode}, {port, 0},
+					{from, self()},
+                                        {mfa, {ssl_test_lib, ReceiveFunction, []}},
+                                        {options, [{active, Active}, {verify, verify_peer},
+                                                   {cacerts, [ServerRoot]} |  
+                                                   proplists:delete(cacerts, ServerConf)]}]),
+    Port  = ssl_test_lib:inet_port(Server),
+    Client = ssl_test_lib:start_client([{node, ClientNode}, {port, Port},
+					{host, Hostname},
+                                        {from, self()},
+                                        {mfa, {ssl_test_lib, ReceiveFunction, []}},
+                                        {options, [{active, Active}, 
+                                                   {verify, verify_peer},
+                                                   {depth, 5},
+                                                   {cacerts,  ServerCas ++ ClientCas} | 
+                                                   proplists:delete(cacerts, ClientConf)]}]),
+    ssl_test_lib:check_result(Server, ok, Client, ok),
+    ssl_test_lib:close(Server),
+    ssl_test_lib:close(Client).
+
 
 %%--------------------------------------------------------------------
 %% Internal functions ------------------------------------------------
