@@ -48,21 +48,27 @@ groups() ->
 
 payload_tests() ->
     [server_echos_passive_small,
+     server_echos_passive_chunk_small,
      server_echos_active_once_small,
      server_echos_active_small,
      client_echos_passive_small,
+     client_echos_passive_chunk_small,
      client_echos_active_once_small,
      client_echos_active_small,
      server_echos_passive_big,
+     server_echos_passive_chunk_big,
      server_echos_active_once_big,
      server_echos_active_big,
      client_echos_passive_big,
+     client_echos_passive_chunk_big,
      client_echos_active_once_big,
      client_echos_active_big,
      server_echos_passive_huge,
+     server_echos_passive_chunk_huge,
      server_echos_active_once_huge,
      server_echos_active_huge,
      client_echos_passive_huge,
+     client_echos_passive_chunk_huge,
      client_echos_active_once_huge,
      client_echos_active_huge,
      client_active_once_server_close].
@@ -109,9 +115,11 @@ end_per_group(GroupName, Config) ->
 
 init_per_testcase(TestCase, Config)
   when TestCase == server_echos_passive_huge;
+       TestCase == server_echos_passive_chunk_huge;
        TestCase == server_echos_active_once_huge;
        TestCase == server_echos_active_huge;
        TestCase == client_echos_passive_huge;
+       TestCase == client_echos_passive_chunk_huge;
        TestCase == client_echos_active_once_huge;
        TestCase == client_echos_active_huge ->
     case erlang:system_info(system_architecture) of
@@ -124,9 +132,11 @@ init_per_testcase(TestCase, Config)
 
 init_per_testcase(TestCase, Config)
   when TestCase == server_echos_passive_big;
+       TestCase == server_echos_passive_chunk_big;
        TestCase == server_echos_active_once_big;
        TestCase == server_echos_active_big;
        TestCase == client_echos_passive_big;
+       TestCase == client_echos_passive_chunk_big;
        TestCase == client_echos_active_once_big;
        TestCase == client_echos_active_big ->
     ct:timetrap({seconds, 60}),
@@ -157,6 +167,22 @@ server_echos_passive_small(Config) when is_list(Config) ->
 
 %%--------------------------------------------------------------------
 
+server_echos_passive_chunk_small() ->
+    [{doc, "Client sends 1000 bytes in passive mode to server, that receives them in chunks, "
+     "sends them back, and closes."}].
+
+server_echos_passive_chunk_small(Config) when is_list(Config) -> 
+    ClientOpts = ssl_test_lib:ssl_options(client_opts, Config),
+    ServerOpts = ssl_test_lib:ssl_options(server_opts, Config),
+    {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
+    %%
+    Data = binary:copy(<<"1234567890">>, 100),
+    server_echos_passive_chunk(
+      Data, ClientOpts, ServerOpts, ClientNode, ServerNode, Hostname).
+
+
+%%--------------------------------------------------------------------
+
 server_echos_active_once_small() ->
     [{doc, "Client sends 1000 bytes in active once mode to server, that receives "
      " them, sends them back, and closes."}].
@@ -199,6 +225,21 @@ client_echos_passive_small(Config) when is_list(Config) ->
     client_echos_passive(
       Data, ClientOpts, ServerOpts, ClientNode, ServerNode, Hostname).
 
+%%--------------------------------------------------------------------
+client_echos_passive_chunk__small() ->
+    [{doc, "Server sends 1000 bytes in passive mode to client, that receives them in chunks, "
+      "sends them back, and closes."}].
+
+client_echos_passive_chunk_small(Config) when is_list(Config) -> 
+    ClientOpts = ssl_test_lib:ssl_options(client_opts, Config),
+    ServerOpts = ssl_test_lib:ssl_options(server_opts, Config),
+    {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
+    %%
+    Data = binary:copy(<<"1234567890">>, 100),
+    client_echos_passive_chunk(
+      Data, ClientOpts, ServerOpts, ClientNode, ServerNode, Hostname).
+
+
 %%--------------------------------------------------------------------
 client_echos_active_once_small() ->
     ["Server sends 1000 bytes in active once mode to client, that receives "
@@ -241,6 +282,19 @@ server_echos_passive_big(Config) when is_list(Config) ->
     Data = binary:copy(<<"1234567890">>, 5000),
     server_echos_passive(
       Data, ClientOpts, ServerOpts, ClientNode, ServerNode, Hostname).
+%%--------------------------------------------------------------------
+server_echos_passive_chunk_big() ->
+    [{doc, "Client sends 50000 bytes to server in passive mode, that receives them, "
+     "sends them back, and closes."}].
+
+server_echos_passive_chunk_big(Config) when is_list(Config) -> 
+    ClientOpts = ssl_test_lib:ssl_options(client_opts, Config),
+    ServerOpts = ssl_test_lib:ssl_options(server_opts, Config),
+    {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
+    %%
+    Data = binary:copy(<<"1234567890">>, 5000),
+    server_echos_passive_chunk(
+      Data, ClientOpts, ServerOpts, ClientNode, ServerNode, Hostname).
 
 %%--------------------------------------------------------------------
 
@@ -286,6 +340,22 @@ client_echos_passive_big(Config) when is_list(Config) ->
     client_echos_passive(
       Data, ClientOpts, ServerOpts, ClientNode, ServerNode, Hostname).
 
+
+%%--------------------------------------------------------------------
+client_echos_passive_chunk_big() ->
+    [{doc, "Server sends 50000 bytes to client in passive mode, that receives them, "
+     "sends them back, and closes."}].
+
+client_echos_passive_chunk_big(Config) when is_list(Config) -> 
+    ClientOpts = ssl_test_lib:ssl_options(client_opts, Config),
+    ServerOpts = ssl_test_lib:ssl_options(server_opts, Config),
+    {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
+    %%
+    Data = binary:copy(<<"1234567890">>, 5000),
+    client_echos_passive_chunk(
+      Data, ClientOpts, ServerOpts, ClientNode, ServerNode, Hostname).
+
+
 %%--------------------------------------------------------------------
 client_echos_active_once_big() ->
     [{doc, "Server sends 50000 bytes to client in active once mode, that receives"
@@ -328,6 +398,20 @@ server_echos_passive_huge(Config) when is_list(Config) ->
     server_echos_passive(
       Data, ClientOpts, ServerOpts, ClientNode, ServerNode, Hostname).
 
+%%--------------------------------------------------------------------
+server_echos_passive_chunk_huge() ->
+    [{doc, "Client sends 500000 bytes to server in passive mode, that receives "
+      " them, sends them back, and closes."}].
+
+server_echos_passive_chunk_huge(Config) when is_list(Config) -> 
+    ClientOpts = ssl_test_lib:ssl_options(client_opts, Config),
+    ServerOpts = ssl_test_lib:ssl_options(server_opts, Config),
+    {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
+    %%
+    Data = binary:copy(<<"1234567890">>, 50000),
+    server_echos_passive_chunk(
+      Data, ClientOpts, ServerOpts, ClientNode, ServerNode, Hostname).
+
 %%--------------------------------------------------------------------
 server_echos_active_once_huge() ->
     [{doc, "Client sends 500000 bytes to server in active once mode, that receives "
@@ -369,7 +453,19 @@ client_echos_passive_huge(Config) when is_list(Config) ->
     Data = binary:copy(<<"1234567890">>, 50000),
     client_echos_passive(
       Data, ClientOpts, ServerOpts, ClientNode, ServerNode, Hostname).
+%%--------------------------------------------------------------------
+client_echos_passive_chunk_huge() ->
+    [{doc, "Server sends 500000 bytes to client in passive mode, that receives "
+     "them, sends them back, and closes."}].
 
+client_echos_passive_chunk_huge(Config) when is_list(Config) -> 
+    ClientOpts = ssl_test_lib:ssl_options(client_opts, Config),
+    ServerOpts = ssl_test_lib:ssl_options(server_opts, Config),
+    {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
+    %%
+    Data = binary:copy(<<"1234567890">>, 50000),
+    client_echos_passive_chunk(
+      Data, ClientOpts, ServerOpts, ClientNode, ServerNode, Hostname).
 %%--------------------------------------------------------------------
 client_echos_active_once_huge() ->
     [{doc, "Server sends 500000 bytes to client in active once mode, that receives "
@@ -442,6 +538,28 @@ server_echos_passive(
     ssl_test_lib:close(Server),
     ssl_test_lib:close(Client).
 
+server_echos_passive_chunk(
+  Data, ClientOpts, ServerOpts, ClientNode, ServerNode, Hostname) ->
+    Length = byte_size(Data),
+    Server =
+        ssl_test_lib:start_server(
+          [{node, ServerNode}, {port, 0},
+           {from, self()},
+           {mfa, {?MODULE, echoer_chunk, [Length]}},
+           {options, [{active, false}, {mode, binary} | ServerOpts]}]),
+    Port = ssl_test_lib:inet_port(Server),
+    Client =
+        ssl_test_lib:start_client(
+          [{node, ClientNode}, {port, Port},
+           {host, Hostname},
+           {from, self()},
+           {mfa, {?MODULE, sender, [Data]}},
+           {options, [{active, false}, {mode, binary} | ClientOpts]}]),
+    %%
+    ssl_test_lib:check_result(Server, ok, Client, ok),
+    %%
+    ssl_test_lib:close(Server),
+    ssl_test_lib:close(Client).
 
 server_echos_active_once(
   Data, ClientOpts, ServerOpts, ClientNode, ServerNode, Hostname) ->
@@ -513,6 +631,31 @@ client_echos_passive(
     ssl_test_lib:close(Server),
     ssl_test_lib:close(Client).
 
+
+client_echos_passive_chunk(
+  Data, ClientOpts, ServerOpts, ClientNode, ServerNode, Hostname) ->
+    Length = byte_size(Data),
+    Server =
+        ssl_test_lib:start_server(
+          [{node, ServerNode}, {port, 0},
+           {from, self()},
+           {mfa, {?MODULE, sender, [Data]}},
+           {options, [{active, false}, {mode, binary} | ServerOpts]}]),
+    Port = ssl_test_lib:inet_port(Server),
+    Client =
+        ssl_test_lib:start_client(
+          [{node, ClientNode}, {port, Port},
+           {host, Hostname},
+           {from, self()},
+           {mfa, {?MODULE, echoer_chunk, [Length]}},
+           {options, [{active, false}, {mode, binary} | ClientOpts]}]),
+    %%
+    ssl_test_lib:check_result(Server, ok, Client, ok),
+    %%
+    ssl_test_lib:close(Server),
+    ssl_test_lib:close(Client).
+
+
 client_echos_active_once(
   Data, ClientOpts, ServerOpts, ClientNode, ServerNode, Hostname) ->
     Length = byte_size(Data),
@@ -574,55 +717,51 @@ client_active_once_server_close(
           [{node, ClientNode}, {port, Port},
            {host, Hostname},
            {from, self()},
-           {mfa, {?MODULE, active_once_recv, [Length]}},
+           {mfa, {ssl_test_lib, active_once_recv, [Length]}},
            {options,[{active, once}, {mode, binary} | ClientOpts]}]),
     %%
     ssl_test_lib:check_result(Server, ok, Client, ok).
 
-send(Socket, Data, Count, Verify) ->
-    send(Socket, Data, Count, <<>>, Verify).
-%%
-send(_Socket, _Data, 0, Acc, _Verify) ->
-    Acc;
-send(Socket, Data, Count, Acc, Verify) ->
+send(_Socket, _Data, 0, _) ->
+    ok;
+send(Socket, Data, Count, RecvEcho) ->
     ok = ssl:send(Socket, Data),
-    NewAcc = Verify(Acc),
-    send(Socket, Data, Count - 1, NewAcc, Verify).
-
+    RecvEcho(),
+    send(Socket, Data, Count - 1, RecvEcho).
 
 send_close(Socket, Data) ->
     ok = ssl:send(Socket, Data),
     ssl:close(Socket).
-   
+
 sender(Socket, Data) ->
     ct:log("Sender recv: ~p~n", [ssl:getopts(Socket, [active])]),
-    <<>> =
-        send(
-          Socket, Data, 100,
-          fun(Acc) -> verify_recv(Socket, Data, Acc) end),
-    ok.
+    send(Socket, Data, 100,
+              fun() -> 
+                      ssl_test_lib:recv_disregard(Socket, byte_size(Data)) 
+              end).
 
 sender_active_once(Socket, Data) ->
     ct:log("Sender active once: ~p~n", [ssl:getopts(Socket, [active])]),
-    <<>> =
-        send(
-          Socket, Data, 100,
-          fun(Acc) -> verify_active_once(Socket, Data, Acc) end),
-    ok.
+    send(Socket, Data, 100,
+         fun() -> 
+                 ssl_test_lib:active_once_disregard(Socket, byte_size(Data)) 
+         end).
 
 sender_active(Socket, Data) ->
     ct:log("Sender active: ~p~n", [ssl:getopts(Socket, [active])]),
-    <<>> =
-        send(
-          Socket, Data, 100,
-          fun(Acc) -> verify_active(Socket, Data, Acc) end),
-    ok.
-
+    send(Socket, Data, 100,
+         fun() -> 
+                 ssl_test_lib:active_disregard(Socket, byte_size(Data)) 
+         end).
 
 echoer(Socket, Size) ->
     ct:log("Echoer recv: ~p~n", [ssl:getopts(Socket, [active])]),
     echo_recv(Socket, Size * 100).
 
+echoer_chunk(Socket, Size) ->
+    ct:log("Echoer recv: ~p~n", [ssl:getopts(Socket, [active])]),
+    echo_recv_chunk(Socket, Size, Size * 100).
+
 echoer_active_once(Socket, Size) ->
     ct:log("Echoer active once: ~p~n", [ssl:getopts(Socket, [active])]),
     echo_active_once(Socket, Size * 100).
@@ -633,108 +772,42 @@ echoer_active(Socket, Size) ->
 
 
 %% Receive Size bytes
+echo_recv(_Socket, 0) ->
+    ok;
 echo_recv(Socket, Size) ->
     {ok, Data} = ssl:recv(Socket, 0),
     ok = ssl:send(Socket, Data),
-    NewSize = Size - byte_size(Data),
-    if
-        0 < NewSize ->
-            echo_recv(Socket, NewSize);
-        0 == NewSize ->
-            ok
-    end.
+    echo_recv(Socket, Size - byte_size(Data)).
 
-%% Verify that received data is SentData, return any superflous data
-verify_recv(Socket, SentData, Acc) ->
-    {ok, NewData} = ssl:recv(Socket, 0),
-    SentSize = byte_size(SentData),
-    NewAcc = <<Acc/binary, NewData/binary>>,
-    NewSize = byte_size(NewAcc),
-    if
-        SentSize < NewSize ->
-            {SentData,Rest} = split_binary(NewAcc, SentSize),
-            Rest;
-        NewSize < SentSize ->
-            verify_recv(Socket, SentData, NewAcc);
-        true ->
-            SentData = NewAcc,
-            <<>>
-    end.
 
 %% Receive Size bytes
+echo_recv_chunk(_Socket, _, 0) ->
+    ok;
+echo_recv_chunk(Socket, ChunkSize, Size) ->
+    {ok, Data} = ssl:recv(Socket, ChunkSize),
+    ok = ssl:send(Socket, Data),
+    echo_recv_chunk(Socket, ChunkSize, Size - ChunkSize).
+
+
+%% Receive Size bytes
+echo_active_once(_Socket, 0) ->
+    ok;
 echo_active_once(Socket, Size) ->
     receive
         {ssl, Socket, Data} ->
             ok = ssl:send(Socket, Data),
             NewSize = Size - byte_size(Data),
             ssl:setopts(Socket, [{active, once}]),
-            if
-                0 < NewSize ->
-                    echo_active_once(Socket, NewSize);
-                0 == NewSize ->
-                    ok
-            end
+            echo_active_once(Socket, NewSize)
     end.
 
-%% Verify that received data is SentData, return any superflous data
-verify_active_once(Socket, SentData, Acc) ->
-    receive
-        {ssl, Socket, Data} ->
-            SentSize = byte_size(SentData),
-            NewAcc = <<Acc/binary, Data/binary>>,
-            NewSize = byte_size(NewAcc),
-            ssl:setopts(Socket, [{active, once}]),
-            if
-                SentSize < NewSize ->
-                    {SentData,Rest} = split_binary(NewAcc, SentSize),
-                    Rest;
-                NewSize < SentSize ->
-                    verify_active_once(Socket, SentData, NewAcc);
-                true ->
-                    SentData = NewAcc,
-                    <<>>
-            end
-    end.
-
-
 %% Receive Size bytes
+echo_active(_Socket, 0) ->
+    ok;
 echo_active(Socket, Size) ->
     receive
         {ssl, Socket, Data} ->
             ok = ssl:send(Socket, Data),
-            NewSize = Size - byte_size(Data),
-            if
-                0 < NewSize ->
-                    echo_active(Socket, NewSize);
-                0 == NewSize ->
-                    ok
-            end
-    end.
-
-%% Verify that received data is SentData, return any superflous data
-verify_active(Socket, SentData, Acc) ->
-    receive
-        {ssl, Socket, Data} ->
-            SentSize = byte_size(SentData),
-            NewAcc = <<Acc/binary, Data/binary>>,
-            NewSize = byte_size(NewAcc),
-            if
-                SentSize < NewSize ->
-                    {SentData,Rest} = split_binary(NewAcc, SentSize),
-                    Rest;
-                NewSize < SentSize ->
-                    verify_active(Socket, SentData, NewAcc);
-                true ->
-                    SentData = NewAcc,
-                    <<>>
-            end
-    end.
-
-active_once_recv(_Socket, 0) ->
-    ok;
-active_once_recv(Socket, N) ->
-    receive 
-	{ssl, Socket, Bytes} ->
-            ssl:setopts(Socket, [{active, once}]),
-            active_once_recv(Socket, N-byte_size(Bytes))
-    end.
+            echo_active(Socket, Size - byte_size(Data))
+    end.    
+        
