@@ -462,7 +462,9 @@ find_upper(Lower, Term, T, Dl, Dd, D, RF, Enc, Str) ->
     case If of
         {_, _, _Dots=0, _} -> % even if Len > T
             If;
-        {_, Len, _, _} when Len =< T, D1 < D orelse D < 0 ->
+        {_, _Len=T, _, _} ->  % increasing the depth is meaningless
+            If;
+        {_, Len, _, _} when Len < T, D1 < D orelse D < 0 ->
 	    find_upper(If, Term, T, D1, Dd2, D, RF, Enc, Str);
         _ ->
 	    search_depth(Lower, If, Term, T, Dl, D1, RF, Enc, Str)
@@ -720,36 +722,47 @@ printable_list(_L, 1, _T, _Enc) ->
     false;
 printable_list(L, _D, T, latin1) when T < 0 ->
     io_lib:printable_latin1_list(L);
-printable_list(L, _D, T, Enc) when T >= 0 ->
-    case slice(L, tsub(T, 2)) of
-        false ->
-            false;
-        {prefix, Prefix} when Enc =:= latin1 ->
-            io_lib:printable_latin1_list(Prefix) andalso {true, Prefix};
-        {prefix, Prefix} ->
-            %% Probably an overestimation.
-            io_lib:printable_list(Prefix) andalso {true, Prefix};
-        all when Enc =:= latin1 ->
-            io_lib:printable_latin1_list(L);
+printable_list(L, _D, T, latin1) when T >= 0 ->
+    N = tsub(T, 2),
+    case printable_latin1_list(L, N) of
         all ->
-            io_lib:printable_list(L)
+            true;
+        0 ->
+            {L1, _} = lists:split(N, L),
+            {true, L1};
+        _NC ->
+            false
+    end;
+printable_list(L, _D, T, _Unicode) when T >= 0 ->
+    N = tsub(T, 2),
+    %% Be careful not to traverse more of L than necessary.
+    try string:slice(L, 0, N) of
+        "" ->
+            false;
+        Prefix ->
+            case is_flat(L, lists:flatlength(Prefix)) of
+                true ->
+                    case string:equal(Prefix, L) of
+                        true ->
+                            io_lib:printable_list(L);
+                        false ->
+                            io_lib:printable_list(Prefix)
+                            andalso {true, Prefix}
+                    end;
+                false ->
+                    false
+            end
+    catch _:_ -> false
     end;
 printable_list(L, _D, T, _Uni) when T < 0->
     io_lib:printable_list(L).
 
-slice(L, N) ->
-    try io_lib:chars_length(L) =< N of
-        true ->
-            all;
-        false ->
-            case string:slice(L, 0, N) of
-                "" ->
-                    false;
-                Prefix ->
-                    {prefix, Prefix}
-            end
-    catch _:_ -> false
-    end.
+is_flat(_L, 0) ->
+    true;
+is_flat([C|Cs], N) when is_integer(C) ->
+    is_flat(Cs, N - 1);
+is_flat(_, _N) ->
+    false.
 
 printable_bin0(Bin, D, T, Enc) ->
     Len = case D >= 0 of
@@ -769,6 +782,8 @@ printable_bin0(Bin, D, T, Enc) ->
           end,
     printable_bin(Bin, Len, D, Enc).
 
+printable_bin(_Bin, 0, _D, _Enc) ->
+    false;
 printable_bin(Bin, Len, D, latin1) ->
     N = erlang:min(20, Len),
     L = binary_to_list(Bin, 1, N),
@@ -819,7 +834,7 @@ printable_bin1(Bin, Start, Len) ->
     end.
 
 %% -> all | integer() >=0. Adopted from io_lib.erl.
-% printable_latin1_list([_ | _], 0) -> 0;
+printable_latin1_list([_ | _], 0) -> 0;
 printable_latin1_list([C | Cs], N) when C >= $\s, C =< $~ ->
     printable_latin1_list(Cs, N - 1);
 printable_latin1_list([C | Cs], N) when C >= $\240, C =< $\377 ->
