@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1996-2017. All Rights Reserved.
+%% Copyright Ericsson AB 1996-2018. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -446,8 +446,8 @@ init_dont_hang(Config) when is_list(Config) ->
 	StartLinkRes = proc_lib:start(?MODULE, init_dont_hang_init, [self()], 1000),
 	StartLinkRes = proc_lib:start(?MODULE, init_dont_hang_init, [self()], 1000, []),
 	ok
-    catch _:Error ->
-	    io:format("Error ~p /= ~p ~n",[erlang:get_stacktrace(), StartLinkRes]),
+    catch _:Error:Stacktrace ->
+	    io:format("Error ~p /= ~p ~n",[Stacktrace, StartLinkRes]),
 	    exit(Error)
     end.
 
@@ -542,16 +542,18 @@ system_terminate(Reason,_Parent,_Deb,_State) ->
 
 
 t_format(_Config) ->
-    error_logger:tty(false),
+    {ok,#{level:=Level}} = logger:get_handler_config(default),
+    logger:set_handler_config(default,level,none),
+    error_logger:add_report_handler(?MODULE, self()),
     try
 	t_format()
     after
-	error_logger:tty(true)
+        error_logger:delete_report_handler(?MODULE),
+        logger:set_handler_config(default,level,Level)
     end,
     ok.
 
 t_format() ->
-    error_logger:add_report_handler(?MODULE, self()),
     Pid = proc_lib:spawn(fun '\x{aaa}t_format_looper'/0),
     HugeData = gb_sets:from_list(lists:seq(1, 100)),
     SomeData1 = list_to_atom([246]),
@@ -584,11 +586,12 @@ t_format() ->
     ok.
 
 t_format_arbitrary(_Config) ->
-    error_logger:tty(false),
+    {ok,#{level:=Level}} = logger:get_handler_config(default),
+    logger:set_handler_config(default,level,none),
     try
         t_format_arbitrary()
     after
-        error_logger:tty(true)
+        logger:set_handler_config(default,level,Level)
     end,
     ok.
 
