@@ -24,7 +24,8 @@
 -include_lib("common_test/include/ct.hrl").
 
 suite() ->
-    [{ct_hooks,[ts_install_cth]}].
+    [{ct_hooks,[ts_install_cth]},
+     {timetrap,{minutes,5}}].
 
 all() -> 
     NoStartStop = [eif,otp_5305,otp_5418,otp_7095,otp_8273,
@@ -35,7 +36,8 @@ all() ->
                  distribution, reconnect, die_and_reconnect,
                  dont_reconnect_after_stop, stop_node_after_disconnect,
                  export_import, otp_5031, otp_6115,
-                 otp_8270, otp_10979_hanging_node, otp_14817],
+                 otp_8270, otp_10979_hanging_node, otp_14817,
+                 local_only],
     case whereis(cover_server) of
         undefined ->
             [coverage,StartStop ++ NoStartStop];
@@ -1742,6 +1744,40 @@ otp_13289(Config) ->
     ok = file:delete(File),
     ok.
 
+local_only(Config) ->
+    ok = file:set_cwd(proplists:get_value(data_dir, Config)),
+
+    %% Trying restricting to local nodes too late.
+    cover:start(),
+    {ok,a} = cover:compile(a),
+    [a] = cover:modules(),
+    {error,too_late} = cover:local_only(),
+    cover:stop(),
+
+    %% Now test local only mode.
+    cover:start(),
+    ok = cover:local_only(),
+    [] = cover:modules(),
+    {ok,a} = cover:compile(a),
+    [a] = cover:modules(),
+    done = a:start(5),
+    {ok, {a,{17,2}}} = cover:analyse(a, coverage, module),
+    {ok, [{{a,exit_kalle,0},{1,0}},
+          {{a,loop,3},{5,1}},
+          {{a,pong,1},{1,0}},
+          {{a,start,1},{6,0}},
+          {{a,stop,1},{0,1}},
+          {{a,trycatch,1},{4,0}}]} =
+        cover:analyse(a, coverage, function),
+
+    %% Make sure that it is not possible to run cover on
+    %% slave nodes.
+    {ok,Name} = test_server:start_node(?FUNCTION_NAME, slave, []),
+    {error,local_only} = cover:start([Name]),
+    test_server:stop_node(Name),
+
+    ok.
+
 %%--Auxiliary------------------------------------------------------------
 
 analyse_expr(Expr, Config) ->
